#!/bin/sh
#
# 3641
# Filename: ArchiveCron.sh
# v3.1
#
## Change log
## Bug  Inititals       Description
## ---  ---------       -----------
## 1907 ME              Set backspace for xterm
## 1925 ME              Implement trap function for cleanup on delete.
## 1994 ME              Add not supported message for Small Form Factor.
## 2060 ME              Change wording on SFF message.
## 2061 ME              Don't display contents of crontab file.
## 2051 ME              Add selection to delete archive schedule.
## 2169 ME              Check for presence of archive.spr before allowing
##			schedule creation.
## 2288 ME              Fix bad label on days of week.
## 2784 SA		Display current archive schedule through cron.
## 3014 SA		Give appropriate message when no schedules have been created.
## 3296 SA ME           Add support to display a profile
## 3297 SA ME		Add support for multiple profiles
## 3460 SA 		Divided ArchiveCron.sh into ArchiveCron.incl containing 
##			all of the functions related to scheduling archive jobs.
## 3500 SA ME		Commented out test for ARCHIVPROFILE default and moved the test
##			into ArchiveCron.incl.
## 3570 ME		Migrate old crontab entries.
## 3641 SA		Add filenames and versions in archive file set in v3.1 going forward.
## 6023 BT		Added check for ARCHIVELOCK.  cleanup now also checks for CREATEDLOCK "boolean" 
##			that states whether the archivelock should be cleaned up by this script.

# 1907
stty sane
stty erase 

. /etc/default/bp.ini 2>/dev/null
. $BPDIR/bin/archive/constants.incl 
# 3296 3297
. /$BPDIR/bin/archive/ArchiveCron.incl
. /$BPDIR/bin/archive/profile_tools.incl
. /$BPDIR/bin/archive/common.incl


QuietFlag="n" # Default to interactive mode

ExitIfArchivingNotSupported

#########################################################################
# cleanUpCron
# 
# End state routine to clean up any temporary files and exit.
#########################################################################
cleanUpCron()
{
	Status=$1
	rm -f $TmpCrontabFile
	rm -f $NewCrontabFile
	stty sane
	removeLock
	exit $Status
}

# 1925 Need to setup trap function in case user hits delete.
trap 'cleanUpCron $?' ERR EXIT TERM INT


# 3570 - Migrate old crontab entries
MigrateOldCrontab

# 2169
#ARCHIVEPROFILE="$BPPRO/archive.spr"
	

# 3296 3297
while getopts 'p:' i; do
	case "$i" in
		p )	filename="$OPTARG"
			if [ ! -f "$BPPRO/$filename" ]
			then
				if [ ! -f $ARCHIVEPROFILE ]
				then
					$ECHO "Cannot locate a profile."
					$ECHO "${0}: Usage - $0 [-p <profile_name>] [-q]"
					exit 1
				else
					ARCHIVEPROFILE="$BPPRO/archive.spr"
				fi
			else
				ARCHIVEPROFILE="$BPPRO/$filename"
			fi
			;;
		* )	$ECHO "${0}: Usage - $0 [-p <profile_name>]"
			exit 1
			;;
	esac
done

if [ $OPTIND -le $# ]; then
	$ECHO "${0}: Usage - $0 [-p <profile_name>]"
	exit 1
fi

if [ -z "$filename" ]
then
	filename="$ARCHIVEPROFILE"
fi

# 3500
# 2169
#if [ ! -f $ARCHIVEPROFILE ]
#then
#	$ECHO "You must first create a profile before creating an Archive Schedule."
#	$ECHO "Press <Enter> to return... \c"
#	read
#	exit 1
#fi
# 3296 3297
RequestUserInput
cleanup $?
