#!/bin/sh
#
# config_iSeries.sh
#
# ME	4623	040208	Interactive script to configure an iSeries client and backup.
# ME	4919	051208	Insert a "press return" so msgs don't flash in motif gui.
# ME	5101	052008	Require an include list on Selective strategy.
# ME	5104	052008	Remove reference to CMC
# ME	5113	052008	wording change on includes/excludes
# ME	5119	052008	add new backup types for on demand backups. disgard the profile after.
# ME	5110	052208	ask for Press Enter if no d2d device found.
# ME	5290	060508	fix misspelling in profile Dialy
#

. /etc/default/bp.ini 2> /dev/null
if [ "$BPDIR" = "" ]
then
	export BPDIR="/usr/bp"
fi
BPBIN=$BPDIR/bin
BPLOG=$BPDIR/logs.dir
iSERIESLOG=$BPLOG/iSeries_config$$.log
BPPRO=$BPDIR/profiles.dir
i5PRO=$BPPRO/iSeries
mkdir -p $i5PRO

ECHO="echo -e"


#########################################################################
#
# cleanup
#
# End state routine to clean up any temporary files and exit.
#
#########################################################################
cleanup()
{
	Status=$1
	# keep the number of iSeries backup logs from growing exponetially
	NumiSeriesLogs=`ls $BPLOG/iSeries_config* | wc -l`
	if [ $NumiSeriesLogs -gt 20 ]
	then
		for log in `ls -tr $BPLOG/iSeries_config*`
		do
			rm -f $log
			NumiSeriesLogs=`expr $NumiSeriesLogs - 1`
			if [ $NumiSeriesLogs -le 20 ]
			then
				break
			fi
		done
	fi
	rm -f $iSERIESLOCK
	stty sane
	exit $Status
}


#########################################################################
#
# profile_create
#
# Setup the profile for the iSeries backup cron job
#
#########################################################################
profile_update()
{
	Section="$1"
	Keyword="$2"
	Value="$3"
	$BPBIN/bputil -p "$Section" "$Keyword" "$Value" $iSERIESPROFILE
}

#########################################################################
#
# CreateiSeriesClient
#
# Take the selected client name and use bpbuild to create the database
# entry for this client.
#
#########################################################################
CreateiSeriesClient()
{
	ClientExists=`$BPBIN/iSeries/iSeries_util.php -e $iSeriesClient`
	if [ "$ClientExists" = "true" ]
	then
		$ECHO "`date`: Detected client $iSeriesClient already in database." >> $iSERIESLOG
	else
		$BPBIN/iSeries/iSeries_util.php -L
		if [ $? -ne 0 ]
		then
			$ECHO "ERROR: License not valid to register client." | tee -a $iSERIESLOG
			$ECHO "Press <Enter>: \c"
			read
			exit 1
		fi
		$BPBIN/bputil --CreateiSeriesClient $iSeriesClient >>$iSERIESLOG 2>&1
		if [ $? -ne 0 ]
		then
			$ECHO "ERROR: Could not create client entry in database for $iSeriesClient." | tee -a $iSERIESLOG
			# 5110
			$ECHO "Press <Enter>: \c"
			read
			exit 1
		fi
		$ECHO "`date`: Created client entry in database for $iSeriesClient." >> $iSERIESLOG
	fi
}

#########################################################################
#
# GetiSeriesSyncBit
#
# Find out if they want to set this client to sync to vault
#
#########################################################################
GetiSeriesSyncBit()
{

	# 5113
	$ECHO "Would you like $iSeriesClient to be synchronized to a vault? (selective backups will not be vaulted)"
	$ECHO "[ y or n - default n ] \c"
	read iSeriesSyncBit

	if [ "$iSeriesSyncBit" = "y" -o "$iSeriesSyncBit" = "Y" ]
	then
		$BPBIN/iSeries/iSeries_util.php -c $iSeriesClient
		if [ $? -ne 0 ]
		then
			$ECHO "ERROR: Could not set requested sync status for $iSeriesClient." | tee -a $iSERIESLOG
			# 5110
			$ECHO "Press <Enter>: \c"
			read
			exit 1
		fi
		$ECHO "`date`: Set sync bit for client $iSeriesClient in database." >> $iSERIESLOG
		profile_update "iSeriesBackup" "SyncBackups" "y"
	elif [ "$iSeriesSyncBit" = "n" -o "$iSeriesSyncBit" = "N" -o "$iSeriesSyncBit" = "" ]
	then
		$ECHO "User does not wish to set sync bit for $iSeriesClient." >> $iSERIESLOG
		profile_update "iSeriesBackup" "SyncBackups" "n"
	else
		clear
		$ECHO "You have entered an invalid response. Please try again."
		GetiSeriesSyncBit
	fi
	return
}

#########################################################################
#
# SetDefaultProfile
#
# Let the user decide if they would like to mark this as the default
# profile.
#
#########################################################################
SetDefaultProfile()
{

	# 5104
	$ECHO "You may choose to set this profile as your default iSeries profile for On-Demand execution"
	$ECHO "from the User Interface - located in the Settings -> System, Update, and Licensing ->"
	$ECHO "Support Toolbox [Advanced] section."
	$ECHO "Set this as default profile? [y or n - default y]: \c"
	read defaultProfile

	if [ "$defaultProfile" != "n" -a "$defaultProfile" != "N" ]
	then
		rm -f $i5PRO/default.spr 2> /dev/null
		ln -s $iSERIESPROFILE $i5PRO/default.spr
	fi
	return
}
#########################################################################
#
# GetiSeriesClientName
#
# Get the name of the iSeries client (must already be setup on network.
#
#########################################################################
GetiSeriesClientName()
{

	$ECHO "Note that the iSeries you wish to backup must already be configured on your network"
	$ECHO "and accessible via DNS or entered into the System's /etc/hosts file."
	$ECHO "Please enter the name of the iSeries client you wish to backup: \c"
	read iSeriesClient

	if [ "$iSeriesClient" = "" ]
	then
		clear
		$ECHO "You must enter a valid client name. Please try again."
		GetiSeriesClientName
	fi
	return
}

#########################################################################
#
# GetiSeriesProfileName
#
# Get the name the user would like to save the profile as.
#
#########################################################################
GetiSeriesProfileName()
{
	$ECHO "Please enter the name you would like to save this profile as: \c"
	read iSeriesProfile

	if [ "$iSeriesProfile" = "" ]
	then
		clear
		$ECHO "You must enter a valid profile name. Please try again."
		GetiSeriesProfileName
	else
		test -s $i5PRO/$iSeriesProfile.spr
		if [ $? = 0 ]
		then
			$ECHO "Profile already exists. Would you like to overwrite it? <y or n - default n>: \c"
			read ANS
			if [ "$ANS" != "y" -a "$ANS" != "Y" ]
			then
				GetiSeriesProfileName
			fi
		fi
	fi
}

#########################################################################
#
# GetUserNameAndPassword
#
# Get the name of the iSeries client (must already be setup on network.
# Store the username in the profile, but hide the password in a
# "dotted" file saved in the profiles directory using the name of the
# client_IPW for the filename.
#
# Restore will have to know about this naming converntion.
# This makes it a little harder to find the password since we are
# being a but unsecure in storing a password.
#
#########################################################################
GetUserNameAndPassword()
{

	$ECHO "In order to connect to the iSeries server, you must supply an"
	$ECHO "authorized username and password for this connection."
	$ECHO "Please enter the username you wish to use to connect: \c"
	read iSeriesUsername

	if [ "$iSeriesUsername" = "" ]
	then
		clear
		$ECHO "You must enter a valid username. Please try again."
		GetUserNameAndPassword
	fi

	profile_update "iSeriesBackup" "UserName" "$iSeriesUsername"

	$ECHO "Please enter the password for the supplied username: \c"
	stty -echo
	read iSeriesPassword
	stty echo
	$ECHO 

	if [ "$iSeriesPassword" = "" ]
	then
		clear
		$ECHO "You must enter a valid password. Please try again."
		GetUserNameAndPassword
	fi

	echo "$iSeriesPassword" > $i5PRO/.${iSeriesProfile}_IPW
	return
}

#########################################################################
#
# GetFiles
#
# Ask for file names one at a time until they enter a 'q'
#
#########################################################################
GetFiles()
{
	ListFile=$1
	filename=""
	while [ "$filename" != "q" ]
	do
		$ECHO "Filename: \c"
		read filename
		if [ "$filename" != "" -a "$filename" != "q" ]
		then
			if [ "${filename:0:1}" != "/" ]
			then
				filename=/$filename
			fi

			$ECHO "$filename" >> $ListFile
		fi
	done
	return
}

#########################################################################
#
# GetExcludeList
#
# Ask for Full Pathnames to Exclude until they type "q"
#
#########################################################################
GetExcludeList()
{
	clear
	# 5113
	$ECHO "You may enter a list of objects (directories, files, libraries) to exclude from"
	$ECHO "your backup. Please specify the complete pathname for each object and enter"
	$ECHO "only one object per line. When you have completed your exclude list, or if you"
	$ECHO "have none, type 'q' to proceed."
	ExclListFileName=$i5PRO/${iSeriesProfile}.excl
	>$ExclListFileName
	GetFiles $ExclListFileName
	if [ -s $ExclListFileName ]
	then
		profile_update "iSeriesBackup" "ExcludeListFile" "$ExclListFileName"
		$ECHO "`date`: Added the following files for exclusion for $iSeriesClient:" >> $iSERIESLOG
		cat $ExclListFileName >> $iSERIESLOG
	fi
}

#########################################################################
#
# GetIncludeList
#
# Ask for Full Pathnames to Includes until they type "q"
#
#########################################################################
GetIncludeList()
{
	clear
	# 5101
	# 5113
	$ECHO "You must enter a list of objects (directories, files, libraries) to include in"
	$ECHO "your Selective backup. Please specify the complete pathname for each object and enter"
	$ECHO "only one object per line. When you have completed your include list, type 'q' to proceed."
	InclListFileName=$i5PRO/${iSeriesProfile}.incl
	>$InclListFileName
	GetFiles $InclListFileName
	if [ -s $InclListFileName ]
	then
		profile_update "iSeriesBackup" "IncludeListFile" "$InclListFileName"
		$ECHO "`date`: Added the following files for inclusion for $iSeriesClient:" >> $iSERIESLOG
		cat $InclListFileName >> $iSERIESLOG
	else
		$ECHO "You cannot specifiy Selective backup type without an inclusion list."
		$ECHO "Press <Enter> to create your list: \c"
		read
		GetIncludeList
	fi
}

# 5119
#########################################################################
#
# GetOnDemandBackupType
#
# Backup types supported: master, incr, sel
#	. master has no --date and no --include options
#	. master may have --exclude option
#	. incremental has --date and may have --exclude
#	. selective has no --date but has --include and/or --exclude
#
#########################################################################
GetOnDemandBackupType()
{

	clear
	$ECHO "Please enter the type of backup you would like to run:"
	$ECHO "1. Master"
	$ECHO "2. Incremental"
	$ECHO "3. Selective"
	$ECHO "Enter Selection: \c"
	read ANS

	if [ "$ANS" = "" ]
	then
		$ECHO "You must enter a valid selection. Please try again."
		GetOnDemandBackupType
	fi

	case "$ANS" in
	"1")
		profile_update "iSeriesBackup" "BackupStrategy" "OnDemandMaster"
		$ECHO "`date`: Backup type set to OnDemandMaster for $iSeriesClient:" >> $iSERIESLOG
		GetExcludeList
	;;
	"2")
		profile_update "iSeriesBackup" "BackupStrategy" "OnDemandIncremental"
		$ECHO "`date`: Backup type set to OnDemandIncremental for $iSeriesClient:" >> $iSERIESLOG
		GetExcludeList
	;;
	"3")
		profile_update "iSeriesBackup" "BackupStrategy" "OnDemandSelective"
		$ECHO "`date`: Backup type set to OnDemandSelective for $iSeriesClient:" >> $iSERIESLOG
		GetIncludeList
		GetExcludeList
	;;
	*)
		$ECHO "You must enter a valid selection. Please try again."
		GetBackupType
	;;
	esac

	return
}

#########################################################################
#
# GetBackupType
#
# Backup types supported: master, incr, sel
#	. master has no --date and no --include options
#	. master may have --exclude option
#	. incremental has --date and may have --exclude
#	. selective has no --date but has --include and/or --exclude
#
#########################################################################
GetBackupType()
{

	clear
	$ECHO "Please enter the type of backups you would like to"
	$ECHO "configure:"
	$ECHO "1. Saturday Master, Incrementals all other days."
	$ECHO "2. Selective every day."
	$ECHO "Enter Selection: \c"
	read ANS

	if [ "$ANS" = "" ]
	then
		$ECHO "You must enter a valid selection. Please try again."
		GetBackupType
	fi

	# 5290
	case "$ANS" in
	"1")
		profile_update "iSeriesBackup" "BackupStrategy" "WeeklyMastDailyIncr"
		$ECHO "`date`: Backup type set to WeeklyMastDailyIn for $iSeriesClient:" >> $iSERIESLOG
		GetExcludeList
	;;
	"2")
		profile_update "iSeriesBackup" "BackupStrategy" "DailySelective"
		$ECHO "`date`: Backup type set to DailySelective for $iSeriesClient:" >> $iSERIESLOG
		GetIncludeList
		GetExcludeList
	;;
	*)
		$ECHO "You must enter a valid selection. Please try again."
		GetBackupType
	;;
	esac

	return
}

#########################################################################
#
# TestiSeriesConnection
#
# backupFTP offers a --test option to allow us to validate the client
# connectitiy settings we have been supplied during this configuration.
#
#########################################################################
TestiSeriesConnection()
{
	export IPW=`cat $i5PRO/.${iSeriesProfile}_IPW`
	$ECHO "`date`: Testing backup configuration for $iSeriesClient with the following command:" >> $iSERIESLOG
	$ECHO "$BPBIN/backupFTP --test --server $iSeriesClient --credentials ${iSeriesUsername}:ENV:IPW" >> $iSERIESLOG
	$BPBIN/backupFTP --test --server "$iSeriesClient" --credentials ${iSeriesUsername}:ENV:IPW >> $iSERIESLOG 2>&1
	RC=$?
	if [ $RC -ne 0 ]
	then
		$ECHO "`date`: The test connection to $iSeriesClient FAILED with return code $RC." >> $iSERIESLOG
		$ECHO "The test connection to $iSeriesClient FAILED with return code $RC."
		$ECHO "Please verify the client name and user/password  credentials you"
		$ECHO "supplied are correct and rerun this configuration utility."
		# 4919
		$ECHO "Press <Enter> to return: \c"
		read
		exit 2
	else
		$ECHO "`date`: The test connection to $iSeriesClient SUCCEEDED." >> $iSERIESLOG
		$ECHO "The test connection to $iSeriesClient SUCCEEDED."
	fi
}

#
# MAIN PROGRAM
#
# iSeries backups will be placed in the Default backup device folder.
# iSeries_util.php -d returns the pathname for the D2D device
# directory.


if [ "$1" != "" ]
then
	SaveProfileName="true"
	OnDemand="true"
fi

if [ "$1" = "-t" ]
then
	iSeriesProfile="tmp$$"
else
	GetiSeriesProfileName
fi

GetiSeriesClientName

iSERIESLOCK=/var/lock/$iSeriesClient

if ! lockfile -r 0 $iSERIESLOCK 2> /dev/null
then
        $ECHO "Processing already in progress for iSeries client $iSeriesClient. Cannot continue."
        $ECHO "`date`: Processing already in progress for iSeries client $iSeriesClient. Cannot continue."  >> $iSERIESLOG
        sleep 5
        exit 1
fi

trap 'cleanup $?' ERR EXIT

# 5119
iSERIESPROFILE=$i5PRO/$iSeriesProfile.spr
> $iSERIESPROFILE

profile_update "iSeriesBackup" "ProfileName" "$iSeriesProfile"

profile_update "iSeriesBackup" "ClientName" "$iSeriesClient" 

DefaultD2DDevice=`$BPBIN/iSeries/iSeries_util.php -d`
if [ ! -d "$DefaultD2DDevice" ]
then
	$ECHO "Cannot locate default D2D device. Cannot continue." | tee -a $iSERIESLOG
	# 5110
	$ECHO "Press <Enter>: \c"
	read
	exit 1
fi

profile_update "iSeriesBackup" "DevicePath" "$DefaultD2DDevice" 

DefaultD2DDeviceName=`$BPBIN/iSeries/iSeries_util.php -n`
if [ "$DefaultD2DDeviceName" = "" ]
then
	$ECHO "Cannot determine default D2D device name. Cannot continue." | tee -a $iSERIESLOG
	# 5110
	$ECHO "Press <Enter>: \c"
	read
	exit 1
fi

profile_update "iSeriesBackup" "DeviceName" "$DefaultD2DDeviceName" 

GetUserNameAndPassword

TestiSeriesConnection

CreateiSeriesClient 

GetiSeriesSyncBit

if [ "$OnDemand" = "true" ]
then
	GetOnDemandBackupType
else
	GetBackupType
	SetDefaultProfile
fi


$ECHO "Your backup profile for $iSeriesClient has been configured."
# 4919
$ECHO "Press <Enter>: \c"
read

if [ "$SaveProfileName" = "true" ]
then
	echo $iSERIESPROFILE > /tmp/.config_profile_name
fi
