#!/usr/bin/php
<?php

/*
This script is used to get the latest information about the ESX/vCenter Inventory and is run every 24 hours
*/

$connections  = array(); /* VMware Connections information */
$associations = array(); /* ESX and vCenter Server association */
$assoc_array  = array();
$uuids        = array(); /* Keep track of ESX and vCenter UUIDs */
$use_cache    = 0;       /* Always run bp_get_vm_info with use_cache = false */

exec('psql -U postgres bpdb --tuples-only -c "select esx_uuid, vcenter_uuid from bp.vmware_connections"', $connections, $retval);
#print_r($connections);

if (!$retval) {
	/* Run bp_get_vm_info only if at least one vCenter/ESX Server is registered to the appliance */
	if (!empty($connections[0])) {

		exec('psql -U postgres bpdb --tuples-only -c "select * from bp.vmware_vcenter_assoc"', $associations, $retval);
		#print_r($associations);

		if (!$retval) {

			/*
			Build the 'assoc_array' consisting of --- [esx_uuid] => [vcenter_uuid] --- pairs
			*/
			foreach ($associations as $temp) {
				$no_space = str_replace(' ', '', $temp); # Remove all spaces
				$uuids = explode("|", $no_space); # Separate ESX and vCenter UUID
				if ( (isset($uuids[0]) && !empty($uuids[0])) && (isset($uuids[1]) && !empty($uuids[1])) ) {
					$assoc_array[$uuids[0]] = $uuids[1];
				}
			}
			#print_r($assoc_array);

			/*
			Check information for each connection
			*/
			foreach ($connections as $temp) {
				$no_space = str_replace(' ', '', $temp);
				$uuids = explode("|", $no_space);

				/* Check if need to call bp_get_vm_info for an ESX Server */
				if (isset($uuids[0]) && !empty($uuids[0])) {
					if (array_key_exists($uuids[0], $assoc_array)) {
						/* 
						This ESX has entry in the vmware_vcenter_assoc table.
						We run bp_get_vm_info for the associated vCenter. 
						So, do not run bp_get_vm_info again for this ESX.
						*/
						echo "Not running bp_get_vm_info\n";
						continue;
					} else {
						/*
						This is a stand alone ESX Server. Hence, doesn't have
						any associated vCenter in our DB.
						*/
						echo "Running bp_get_vm_info for a stand-alone ESX Server\n";
						bp_bypass_cookie(3);
						$val = bp_get_vm_info($uuids[0], '', $use_cache);
						if ($val == false || $val == NULL) {
							echo "Error is: " . bp_error() . "\n";
							bp_destroy_cookie();
							echo "Failed to get information for ESX Server with UUID $uuids[0]\n";
						} else {
							bp_destroy_cookie();
						}
						continue;
					}
				}

				/* Check if need to call bp_get_vm_info for a vCenter Server */
				if (isset($uuids[1]) && !empty($uuids[1])) {
					echo "Running bp_get_vm_info for a vCenter Server\n";
					bp_bypass_cookie(3);
					$val = bp_get_vm_info($uuids[1], '', $use_cache);
					if ($val == false || $val == NULL) {
						echo "Error is: " . bp_error() . "\n";
						bp_destroy_cookie();
						echo "Failed to get information for ESX Server with UUID $uuids[0]\n";
					} else {
						bp_destroy_cookie();
					}
					continue;
				}
			}

		} /* End if for vmware_vcenter_assoc check */
		else {
			echo "exec() for checking VMware association information failed\n";
		}

	} /* End if for no ESX/vCenter Servers registered to the appliance */
	else {
		echo "No ESX/vCenter Servers are registered to this appliance\n";
	}
} else {
	echo "exec() for getting VMware connections information failed\n";
}

echo "\n";

?>
