<?php
/**
 * Class to hold common functions
 * User: Sonja Barton
 * Date: 7/18/14
 */

// in progress
class Functions {
    private $BP;

    public function __construct($BP = NULL){
		$this->BP = $BP;
    }

	// Return a complete list of system ids and names for looping.
    public function selectSystems($sid = false, $include_non_managed_replicating_systems = true)
    {
        $filter = array();
        if( $sid !== false )
        {
            $filter['system_ids'] = $sid;
        }
        elseif ( $include_non_managed_replicating_systems === false )
        {
            $filter['system_types'] = array('not_non-managed_replication_source');
        }

        $systems = $this->BP->get_systems( false, $filter );

        // This logic is temporary until the fix can be made in the core
        foreach ($systems as $id => $name)
        {
            if (strstr($name, '.dpu'))
            {
                unset($systems[$id]);
                break;
            }
        }

        return $systems;
    }

    // Return a list of replicating system ids and names for looping.
    public function selectReplicatingSystems( $sid = false )
    {
        return $this->getSystems( false, $sid, false, false, true, true, false );
    }

    // Returns the name of a given systemID
    public function getSystemNameFromID( $sid )
    {
        $name = false;
        if ( is_numeric($sid) )
        {
            $systems = $this->getSystems( false, $sid );
            $name = $systems[$sid];
        }
        return $name;
    }

    // Returns a list of Systems
    // if getDetails is false, only name and id are returned, else all of the info from bp_get_system_info is returned for each system
    // $systemIDs, $customerIDs, and $locationIDs accept either a single integer or an array of integers
    // $systemNames, $customerNames, and $locationNames accept either a single string or an array of strings
    public function getSystems( $getDetails = false,
                                $systemIDs = false,
                                $includeLocalSystem = true,
                                $includeManagedSystems = true,
                                $includeManagedReplicatingSystems = true,
                                $includeNonManagedReplicatingSystems = true,
                                $includeVaultingSystems = true,
                                $customerIDs = NULL,
                                $locationIDs = NULL,
                                $systemNames = NULL,
                                $customerNames = NULL,
                                $locationNames = NULL )
    {
        $filter = array();
        if( $systemIDs !== false )
        {
            $filter['system_ids'] = $systemIDs;
        }
        if ( $includeLocalSystem !== true or $includeManagedSystems !== true or $includeManagedReplicatingSystems !== true or $includeNonManagedReplicatingSystems !== true or $includeVaultingSystems !== true )
        {
            $systemTypes = array();
            if ( $includeLocalSystem === true )
            {
                $systemTypes[] = 'local';
            }
            if ( $includeManagedSystems === true )
            {
                $systemTypes[] = 'managed';
            }
            if ( $includeManagedReplicatingSystems === true )
            {
                $systemTypes[] = 'managed replicating';
            }
            if ( $includeNonManagedReplicatingSystems === true )
            {
                $systemTypes[] = 'non-managed replicating';
            }
            if ( $includeVaultingSystems === true )
            {
                $systemTypes[] = 'vaulting';
            }
            $filter['system_types'] = $systemTypes;
        }
        if ( $systemNames !== NULL )
        {
            $filter['system_names'] = $systemNames;
        }
        if ( $customerIDs !== NULL )
        {
            $filter['customer_ids'] = $customerIDs;
        }
        if ( $customerNames !== NULL )
        {
            $filter['customer_names'] = $customerNames;
        }
        if ( $locationIDs !== NULL )
        {
            $filter['location_ids'] = $locationIDs;
        }
        if ( $locationNames !== NULL )
        {
            $filter['location_names'] = $locationNames;
        }

        return $this->BP->get_systems( $getDetails, $filter );
    }

    function formatTimeDelta($delta)
    {
        $text = '';
        if ($delta > 0) {
            if ($hours = floor($delta/3600)) {
                $delta = $delta % 3600;
            }
            if ($minutes = floor($delta/60)) {
                $delta = $delta % 60;
            }
            $seconds = floor($delta);
            $text = sprintf("%02d:%02d:%02d", $hours, $minutes, $seconds);
        }
        return $text;
    }
	
	const DATE_FORMAT_US = Constants::DATE_FORMAT_US;
	const TIME_FORMAT_12H = Constants::TIME_FORMAT_12H;
	const DATE_TIME_FORMAT_US = Constants::DATE_TIME_FORMAT_US;
	
    function formatDate($timestamp)
    {
		return date(Constants::DATE_FORMAT_US, $timestamp);
    }

    function formatDateTime($timestamp)
    {
		return date(Constants::DATE_TIME_FORMAT_US, $timestamp);
    }

    function formatDateTime24Hour($timestamp)
    {
        return date(Constants::DATE_TIME_FORMAT_24H, $timestamp);
    }

    // Returns the Application Type of a Given Application ID or false on failure
    public function getApplicationTypeFromApplictionID( $app_id )
    {
        $app_type = false;
        switch ($app_id)
        {
            case Constants::APPLICATION_ID_FILE_LEVEL:
                $app_type = Constants::APPLICATION_TYPE_NAME_FILE_LEVEL;
                break;
            case Constants::APPLICATION_ID_EXCHANGE_2003:
            case Constants::APPLICATION_ID_EXCHANGE_2007:
            case Constants::APPLICATION_ID_EXCHANGE_2010:
            case Constants::APPLICATION_ID_EXCHANGE_2013:
                $app_type = Constants::APPLICATION_TYPE_NAME_EXCHANGE;
                break;
            case Constants::APPLICATION_ID_SQL_SERVER_2005:
            case Constants::APPLICATION_ID_SQL_SERVER_2008:
            case Constants::APPLICATION_ID_SQL_SERVER_2008_R2:
            case Constants::APPLICATION_ID_SQL_SERVER_2012:
            case Constants::APPLICATION_ID_SQL_SERVER_2014:
                $app_type = Constants::APPLICATION_TYPE_NAME_SQL_SERVER;
                break;
            case Constants::APPLICATION_ID_ARCHIVE:
                $app_type = Constants::APPLICATION_TYPE_NAME_ARCHIVE;
                break;
            case Constants::APPLICATION_ID_VMWARE:
                $app_type = Constants::APPLICATION_TYPE_NAME_VMWARE;
                break;
            case Constants::APPLICATION_ID_HYPER_V_2008_R2:
            case Constants::APPLICATION_ID_HYPER_V_2012:
                $app_type = Constants::APPLICATION_TYPE_NAME_HYPER_V;
                break;
            case Constants::APPLICATION_ID_SYSTEM_METADATA:
                $app_type = Constants::APPLICATION_TYPE_NAME_SYSTEM_METADATA;
                break;
            case Constants::APPLICATION_ID_ORACLE_10:
            case Constants::APPLICATION_ID_ORACLE_11:
            case Constants::APPLICATION_ID_ORACLE_12:
                $app_type = Constants::APPLICATION_TYPE_NAME_ORACLE;
                break;
            case Constants::APPLICATION_ID_SHAREPOINT_2007:
            case Constants::APPLICATION_ID_SHAREPOINT_2010:
            case Constants::APPLICATION_ID_SHAREPOINT_2013:
                $app_type = Constants::APPLICATION_TYPE_NAME_SHAREPOINT;
                break;
            case Constants::APPLICATION_ID_UCS_SERVICE_PROFILE:
                $app_type = Constants::APPLICATION_TYPE_NAME_UCS_SERVICE_PROFILE;
                break;
            case Constants::APPLICATION_ID_VOLUME:
                $app_type = Constants::APPLICATION_TYPE_NAME_VOLUME;
                break;
            case Constants::APPLICATION_ID_XEN:
                $app_type = Constants::APPLICATION_TYPE_NAME_XEN;
                break;
        }
        return $app_type;
    }

    public function getBackupTypeCoreName($backupType, $appType) {
        switch($appType) {
            case Constants::APPLICATION_TYPE_NAME_VMWARE:
                switch($backupType) {
                    case Constants::BACKUP_DISPLAY_TYPE_FULL:
                        $backupType = "VMware Full";
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL:
                        $backupType = "VMware Incremental";
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL:
                        $backupType = "VMware Differential";
                        break;
                }
                break;
            case Constants::APPLICATION_TYPE_NAME_HYPER_V:
                    switch($backupType) {
                        case Constants::BACKUP_DISPLAY_TYPE_FULL:
                            $backupType = "Hyper-V Full";
                            break;
                        case Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL:
                            $backupType = "Hyper-V Incremental";
                            break;
                        case Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL:
                            $backupType = "Hyper-V Differential";
                            break;
                    }
                break;
            case Constants::APPLICATION_TYPE_NAME_ORACLE:
                switch($backupType) {
                    case Constants::BACKUP_DISPLAY_TYPE_FULL:
                        $backupType = "Oracle Full";
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL:
                        $backupType = "Oracle Incremental";
                        break;
                }
                break;
            case Constants::APPLICATION_TYPE_NAME_SQL_SERVER:
                switch($backupType) {
                    case Constants::BACKUP_DISPLAY_TYPE_FULL:
                        $backupType = "SQL Full";
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL:
                        $backupType = "SQL Differential";
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_TRANSACTION:
                        $backupType = "SQL Transaction";
                        break;
                }
                break;
            case Constants::APPLICATION_TYPE_NAME_SHAREPOINT:
                switch($backupType) {
                    case Constants::BACKUP_DISPLAY_TYPE_FULL:
                        $backupType = "SharePoint Full";
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL:
                        $backupType = "SharePoint Differential";
                        break;
                }
                break;
            case Constants::APPLICATION_TYPE_NAME_EXCHANGE:
                switch($backupType) {
                    case Constants::BACKUP_DISPLAY_TYPE_FULL:
                        $backupType = "Exchange Full";
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL:
                        $backupType = "Exchange Incremental";
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL:
                        $backupType = "Exchange Differential";
                        break;
                }
                break;
            case Constants::APPLICATION_TYPE_NAME_FILE_LEVEL:
                if($backupType == Constants::BACKUP_DISPLAY_TYPE_FULL) {
                    $backupType = "Master";
                }
                break;
            case Constants::APPLICATION_TYPE_NAME_XEN:
                switch($backupType) {
                    case Constants::BACKUP_DISPLAY_TYPE_FULL:
                        $backupType = "Xen Full";
                        break;
                }
                break;
        }
        return $backupType;
    }

    public function getBackupTypeCoreNameBackupNow($backupType, $appType) {
        switch($appType) {
            case Constants::APPLICATION_TYPE_NAME_VMWARE:
                switch($backupType) {
                    case Constants::BACKUP_DISPLAY_TYPE_FULL:
                        $backupType = Constants::BACKUP_TYPE_VMWARE_FULL;
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL:
                        $backupType = Constants::BACKUP_TYPE_VMWARE_INCREMENTAL;
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL:
                        $backupType = Constants::BACKUP_TYPE_VMWARE_DIFFERENTIAL;
                        break;
                }
                break;
            case Constants::APPLICATION_TYPE_NAME_HYPER_V:
                switch($backupType) {
                    case Constants::BACKUP_DISPLAY_TYPE_FULL:
                        $backupType = Constants::BACKUP_TYPE_HYPER_V_FULL;
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL:
                        $backupType = Constants::BACKUP_TYPE_HYPER_V_INCREMENTAL;
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL:
                        $backupType = Constants::BACKUP_TYPE_HYPER_V_DIFFERENTIAL;
                        break;
                }
                break;
            case Constants::APPLICATION_TYPE_NAME_ORACLE:
                switch($backupType) {
                    case Constants::BACKUP_DISPLAY_TYPE_FULL:
                        $backupType = Constants::BACKUP_TYPE_ORACLE_FULL;
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL:
                        $backupType = Constants::BACKUP_TYPE_ORACLE_INCR;
                        break;
                }
                break;
            case Constants::APPLICATION_TYPE_NAME_SQL_SERVER:
                switch($backupType) {
                    case Constants::BACKUP_DISPLAY_TYPE_FULL:
                        $backupType = Constants::BACKUP_TYPE_MSSQL_FULL;
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL:
                        $backupType = Constants::BACKUP_TYPE_MSSQL_DIFFERENTIAL;
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_TRANSACTION:
                        $backupType = Constants::BACKUP_TYPE_MSSQL_TRANSACTION;
                        break;
                }
                break;
            case Constants::APPLICATION_TYPE_NAME_SHAREPOINT:
                switch($backupType) {
                    case Constants::BACKUP_DISPLAY_TYPE_FULL:
                        $backupType = Constants::BACKUP_TYPE_SHAREPOINT_FULL;
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL:
                        $backupType = Constants::BACKUP_TYPE_SHAREPOINT_DIFF;
                        break;
                }
                break;
            case Constants::APPLICATION_TYPE_NAME_EXCHANGE:
                switch($backupType) {
                    case Constants::BACKUP_DISPLAY_TYPE_FULL:
                        $backupType = Constants::BACKUP_TYPE_EXCHANGE_FULL;
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL:
                        $backupType = Constants::BACKUP_TYPE_EXCHANGE_INCREMENTAL;
                        break;
                    case Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL:
                        $backupType = Constants::BACKUP_TYPE_EXCHANGE_DIFFERENTIAL;
                        break;
                }
                break;
            case Constants::APPLICATION_TYPE_NAME_FILE_LEVEL:
                if($backupType == Constants::BACKUP_DISPLAY_TYPE_FULL) {
                    $backupType = "master";
                }
                break;
            case Constants::APPLICATION_TYPE_NAME_XEN:
                switch($backupType) {
                    case Constants::BACKUP_DISPLAY_TYPE_FULL:
                        $backupType = Constants::BACKUP_TYPE_XEN_FULL;
                        break;
                }
                break;
        }
        return $backupType;
    }

    public function getBackupTypeDisplayName($backupType, $bSynthesized = false)
    {
        $backupDisplayName = '';
        switch($backupType)
        {
            case Constants::BACKUP_TYPE_MASTER:
            case Constants::BACKUP_TYPE_MSSQL_FULL:
            case Constants::BACKUP_TYPE_MSSQL_FULL_ALT:
            case Constants::BACKUP_TYPE_EXCHANGE_FULL:
            case Constants::BACKUP_TYPE_LEGACY_MSSQL_FULL:
            case Constants::BACKUP_TYPE_VMWARE_FULL:
            case Constants::BACKUP_TYPE_HYPER_V_FULL:
            case Constants::BACKUP_TYPE_HYPER_V_FULL_ALT:
            case Constants::BACKUP_TYPE_ORACLE_FULL:
            case Constants::BACKUP_TYPE_SHAREPOINT_FULL:
            case Constants::BACKUP_TYPE_UCS_SERVICE_PROFILE_FULL:
            case Constants::BACKUP_TYPE_NDMP_FULL:
            case Constants::BACKUP_TYPE_XEN_FULL:
            case Constants::BACKUP_TYPE_SECURESYNC_MASTER:
            case Constants::BACKUP_TYPE_SECURESYNC_MSSQL_FULL:
            case Constants::BACKUP_TYPE_SECURESYNC_EXCHANGE_FULL:
            case Constants::BACKUP_TYPE_SECURESYNC_VMWARE_FULL:
            case Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_FULL:
            case Constants::BACKUP_TYPE_SECURESYNC_ORACLE_FULL:
            case Constants::BACKUP_TYPE_SECURESYNC_SHAREPOINT_FULL:
            case Constants::BACKUP_TYPE_SECURESYNC_UCS_SERVICE_PROFILE_FULL:
            case Constants::BACKUP_TYPE_SECURESYNC_NDMP_FULL:
            case Constants::BACKUP_TYPE_SECURESYNC_XEN_FULL:
            case Constants::BACKUP_TYPE_SATORI_REPLICATION_VMWARE_FULL:
            case Constants::BACKUP_TYPE_SATORI_REPLICATION_HV_FULL:
            case Constants::BACKUP_TYPE_SATORI_REPLICATION_SQL_FULL:
            case Constants::BACKUP_TYPE_SATORI_REPLICATION_EXCHANGE_FULL:
                $backupDisplayName = Constants::BACKUP_DISPLAY_TYPE_FULL;
                break;
            case Constants::BACKUP_TYPE_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_MSSQL_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_MSSQL_DIFFERENTIAL_ALT:
            case Constants::BACKUP_TYPE_EXCHANGE_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_LEGACY_MSSQL_DIFF:
            case Constants::BACKUP_TYPE_VMWARE_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_HYPER_V_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_HYPER_V_DIFFERENTIAL_ALT:
            case Constants::BACKUP_TYPE_SHAREPOINT_DIFF:
            case Constants::BACKUP_TYPE_SHAREPOINT_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_NDMP_DIFF:
            case Constants::BACKUP_TYPE_SECURESYNC_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_SECURESYNC_MSSQL_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_SECURESYNC_EXCHANGE_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_SECURESYNC_VMWARE_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_SECURESYNC_SHAREPOINT_DIFF:
            case Constants::BACKUP_TYPE_SECURESYNC_NDMP_DIFF:
            case Constants::BACKUP_TYPE_SATORI_REPLICATION_VMWARE_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_SATORI_REPLICATION_HV_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_SATORI_REPLICATION_SQL_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_SATORI_REPLICATION_EXCHANGE_DIFFERENTIAL:
            $backupDisplayName = Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL;
                break;
            case Constants::BACKUP_TYPE_INCREMENTAL:
            case Constants::BACKUP_TYPE_EXCHANGE_INCREMENTAL:
            case Constants::BACKUP_TYPE_VMWARE_INCREMENTAL:
            case Constants::BACKUP_TYPE_HYPER_V_INCREMENTAL:
            case Constants::BACKUP_TYPE_HYPER_V_INCREMENTAL_ALT:
            case Constants::BACKUP_TYPE_ORACLE_INCR:
            case Constants::BACKUP_TYPE_NDMP_INCR:
            case Constants::BACKUP_TYPE_SECURESYNC_INCREMENTAL:
            case Constants::BACKUP_TYPE_SECURESYNC_EXCHANGE_INCREMENTAL:
            case Constants::BACKUP_TYPE_SECURESYNC_VMWARE_INCREMENTAL:
            case Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_INCREMENTAL:
            case Constants::BACKUP_TYPE_SECURESYNC_ORACLE_INCR:
            case Constants::BACKUP_TYPE_SECURESYNC_NDMP_INCR:
            case Constants::BACKUP_TYPE_SATORI_REPLICATION_VMWARE_INCREMENTAL:
            case Constants::BACKUP_TYPE_SATORI_REPLICATION_HV_INCREMENTAL:
            case Constants::BACKUP_TYPE_SATORI_REPLICATION_SQL_INCREMENTAL:
            case Constants::BACKUP_TYPE_SATORI_REPLICATION_EXCHANGE_INCREMENTAL:
                $backupDisplayName = Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL;
                break;
            case Constants::BACKUP_TYPE_BAREMETAL:
                $backupDisplayName = Constants::BACKUP_DISPLAY_TYPE_BAREMETAL;
                break;
            case Constants::BACKUP_TYPE_SELECTIVE:
                $backupDisplayName = Constants::BACKUP_DISPLAY_TYPE_SELECTIVE;
                break;
            case Constants::BACKUP_TYPE_MSSQL_TRANSACTION:
            case Constants::BACKUP_TYPE_MSSQL_TRANSACTION_ALT:
            case Constants::BACKUP_TYPE_LEGACY_MSSQL_TRANS:
                $backupDisplayName = Constants::BACKUP_DISPLAY_TYPE_TRANSACTION;
                break;
            case Constants::BACKUP_TYPE_RESTORE:
            case Constants::BACKUP_TYPE_INTEGRATED_BM_RESTORE:
            case Constants::BACKUP_TYPE_VIRTUAL_RESTORE:
                $backupDisplayName = Constants::BACKUP_DISPLAY_TYPE_RESTORE;
                break;
            case Constants::BACKUP_COPY_ARCHIVE_RESTORE:
                $backupDisplayName = Constants::BACKUP_COPY_DISPLAY_ARCHIVE_RESTORE;
                break;
            default:
                $backupDisplayName = $backupType;
                break;
        }

        if ($backupDisplayName == "Full" and $bSynthesized == true){
            $backupDisplayName = "Synthetic Full";
        }
        return $backupDisplayName;
    }

    // returns true is $backupType is a securesync type
    public function isBackupTypeSecuresync($backupType)
{
    $isSecuresync = false;
    switch($backupType)
    {
        case Constants::BACKUP_TYPE_SECURESYNC_MASTER:
        case Constants::BACKUP_TYPE_SECURESYNC_DIFFERENTIAL:
        case Constants::BACKUP_TYPE_SECURESYNC_INCREMENTAL:
        case Constants::BACKUP_TYPE_SECURESYNC_BAREMETAL:
        case Constants::BACKUP_TYPE_SECURESYNC_DPU_STATE:
        case Constants::BACKUP_TYPE_SECURESYNC_LOCAL_DIRECTORY:
        case Constants::BACKUP_TYPE_SECURESYNC_MS_SQL:
        case Constants::BACKUP_TYPE_SECURESYNC_EXCHANGE:
        case Constants::BACKUP_TYPE_SECURESYNC_MSSQL_FULL:
        case Constants::BACKUP_TYPE_SECURESYNC_MSSQL_DIFFERENTIAL:
        case Constants::BACKUP_TYPE_SECURESYNC_MSSQL_TRANSACTION:
        case Constants::BACKUP_TYPE_SECURESYNC_EXCHANGE_FULL:
        case Constants::BACKUP_TYPE_SECURESYNC_EXCHANGE_DIFFERENTIAL:
        case Constants::BACKUP_TYPE_SECURESYNC_EXCHANGE_INCREMENTAL:
        case Constants::BACKUP_TYPE_SECURESYNC_VMWARE_FULL:
        case Constants::BACKUP_TYPE_SECURESYNC_VMWARE_DIFFERENTIAL:
        case Constants::BACKUP_TYPE_SECURESYNC_VMWARE_INCREMENTAL:
        case Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_FULL:
        case Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_INCREMENTAL:
        case Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_DIFFERENTIAL:
        case Constants::BACKUP_TYPE_SECURESYNC_ORACLE_FULL:
        case Constants::BACKUP_TYPE_SECURESYNC_ORACLE_INCR:
        case Constants::BACKUP_TYPE_SECURESYNC_SHAREPOINT_FULL:
        case Constants::BACKUP_TYPE_SECURESYNC_SHAREPOINT_DIFF:
        case Constants::BACKUP_TYPE_SECURESYNC_SYSTEM_METADATA:
        case Constants::BACKUP_TYPE_SECURESYNC_UCS_SERVICE_PROFILE_FULL:
        case Constants::BACKUP_TYPE_SECURESYNC_NDMP_FULL:
        case Constants::BACKUP_TYPE_SECURESYNC_NDMP_DIFF:
        case Constants::BACKUP_TYPE_SECURESYNC_NDMP_INCR:
        case Constants::BACKUP_TYPE_SECURESYNC_XEN_FULL:
            $isSecuresync = true;
            break;
    }
    return $isSecuresync;
}

    public function isBackupFileLevel($backupType){
        $isFileLevel = false;

        switch($backupType){
            case Constants::BACKUP_TYPE_MASTER:
            case Constants::BACKUP_TYPE_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_INCREMENTAL:
            case Constants::BACKUP_TYPE_SELECTIVE:

            $isFileLevel = true;
                break;
        }
        return $isFileLevel;
    }

    public function isBackupHyperV($backupType){
        $isHyperV = false;

        switch($backupType){
            case Constants::BACKUP_TYPE_HYPER_V_FULL:
            case Constants::BACKUP_TYPE_HYPER_V_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_HYPER_V_INCREMENTAL:

            $isHyperV = true;
                break;
        }
        return $isHyperV;
    }

    public function isBackupExchange($backupType){
        $isExchange = false;

        switch($backupType){
            case Constants::BACKUP_TYPE_EXCHANGE_FULL:
            case Constants::BACKUP_TYPE_EXCHANGE_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_EXCHANGE_INCREMENTAL:

            $isExchange = true;
                break;
        }
        return $isExchange;
    }

    public function isBackupSQL($backupType){
        $isSQL = false;

        switch($backupType){
            case Constants::BACKUP_TYPE_MSSQL_FULL:
            case Constants::BACKUP_TYPE_MSSQL_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_MSSQL_TRANSACTION:

            $isSQL = true;
                break;
        }
        return $isSQL;
    }

    public function isBackupVMWare($backupType){
        $isVMWare = false;

        switch($backupType){
            case Constants::BACKUP_TYPE_VMWARE_FULL:
            case Constants::BACKUP_TYPE_VMWARE_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_VMWARE_INCREMENTAL:

            $isVMWare = true;
                break;
        }
        return $isVMWare;
    }

    public function isBackupXen($backupType){
        $isXen = false;

        switch($backupType){
            case Constants::BACKUP_TYPE_XEN_FULL:

                $isXen = true;
                break;
        }
        return $isXen;
    }

    public function isBackupVirtualRestore($backupType){
        $isVirtualRestore = false;

        switch($backupType){
            case Constants::BACKUP_TYPE_VIRTUAL_RESTORE:

            $isVirtualRestore = true;
                break;
        }
        return $isVirtualRestore;
    }

    function getBackupTypeString($backupField, $bSynthesized = false)
    {
        $backupType = $backupField;
        switch ($backupField) {
            case "master":
            case "securesync master":
                $backupType = "Full";
                break;
            case "differential":
            case "securesync differential":
                $backupType = "Differential";
                break;
            case "incremental":
            case "securesync incremental":
                $backupType = "Incremental";
                break;
            case "selective":
                $backupType = "Selective";
                break;
            case "baremetal":
            case "securesync baremetal":
                $backupType = "BareMetal";
                break;
            case "securesync mssql full":
            case "mssql full":
                $backupType = "SQL Full";
                break;
            case "securesync mssql diff":
            case "mssql differential":
                $backupType = "SQL Differential";
                break;
            case "securesync mssql trans":
            case "mssql transaction":
                $backupType = "SQL Transaction";
                break;
            case "legacy mssql full":
            case "legacy mssql diff":
            case "legacy mssql trans":
                $backupType = "SQL";
                break;
            case "securesync msexch full":
            case "exchange full":
                $backupType = "Exchange Full";
                break;
            case "securesync msexch diff":
            case "exchange differential":
                $backupType = "Exchange Differential";
                break;
            case "securesync msexch incr":
            case "exchange incremental":
                $backupType = "Exchange Incremental";
                break;
            case "securesync vmware full":
            case "securesync esx vm full":
            case "vmware full":
                $backupType = "Full";
                break;
            case "securesync xen full":
            case "xen full":
                $backupType = "Full";
                break;
            case "securesync vmware diff":
            case "securesync esx vm diff":
            case "vmware differential":
                $backupType = "Differential";
                break;
            case "securesync vmware incr":
            case "securesync esx vm incr":
            case "vmware incremental":
                $backupType = "Incremental";
                break;
            case "vmware configuration":		// only used on restore for first job to restore metadata
                $backupType = "VMware Configuration";
                break;
            case "securesync hyperv full":
            case "securesync hyper-v full":		// operation progress string
            case "hyperv full":
                $backupType = "Full";
                break;
            case "securesync hyperv incr":
            case "securesync hyper-v incr":		// operation progress string
            case "hyperv incremental":
                $backupType = "Incremental";
                break;
            case "securesync hyperv diff":
            case "securesync hyper-v diff":		// operation progress string
            case "hyperv differential":
                $backupType = "Differential";
                break;
            case "restore":
                $backupType = "Restore";
                break;
            case "virtual restore":
                $backupType = "Virtual Restore";
                break;
            case "verify":
                $backupType = "Verify";
                break;
            case "securesync SQL":
            case "securesync sql":
                $backupType = "SQL";
                break;
            case "localdir":
            case "securesync localdir":
                $backupType = "Local Directory";
                break;
            case "securesync exchange":
                $backupType = "Exchange";
                break;
            case "securesync dpustate":
                $backupType = "DPUstate";
                break;
            case "securesync system metadata":
            case "system metadata":
                $backupType = "System Metadata";
                break;
            case "archive":
                $backupType = "Backup Copy";
                break;
            case "archive restore":
                $backupType = "Import";
                break;
            case "securesync oracle full":
            case "oracle full":
                $backupType = "Oracle Full";
                break;
            case "securesync oracle incr":
            case "oracle incr":
                $backupType = "Oracle Incremental";
                break;
            case "securesync sharepoint full":
            case "sharepoint full":
                $backupType = "SharePoint Full";
                break;
            case "securesync sharepoint diff":
            case "sharepoint diff":
                $backupType = "SharePoint Differential";
                break;
            case "securesync ucs service profile full":
            case "ucs service profile full":
                $backupType = "UCS Service Profile Full";
                break;
            case "integrated bm restore":
                $backupType = "Integrated Bare Metal Recovery";
                break;
            case "securesync ndmp full":
            case "ndmp full":
                $backupType = "NDMP Full";
                break;
            case "securesync ndmp diff":
            case "ndmp diff":
                $backupType = "NDMP Differential";
                break;
            case "securesync ndmp incr":
            case "ndmp incr":
                $backupType = "NDMP Incremental";
                break;

        }

        if ($backupType == "Full" and $bSynthesized == true){
            $backupType = "Synthetic Full";
        }
        return $backupType;
    }

    public function getArchiveTypes() {
        return array("master", "differential", "incremental", "selective", "baremetal", "vmware full", "vmware differential", "vmware incremental",
            "hyperv full", "hyperv incremental", "hyperv differential");
    }

    public function getArchiveTypesByDisplayType($type) {
        switch($type) {
            case 'Full':
                $types = array("master", "vmware full", "hyperv full");
                break;
            case 'Incremental':
                $types = array("incremental", "vmware incremental", "hyperv incremental");
                break;
            case 'Differential':
                $types = array("differential", "vmware differential", "hyperv differential");
                break;
            case 'Selective':
                $types = array("selective");
                break;
            case 'Bare Metal':
                $types = array("baremetal");
                break;
        }
        return $types;
    }

    //takes a date in format mm/dd/yyyy hh:mm:ssAM or mm/dd/yyyy and converts to unix timestamp
    public function dateToTimestamp($formattedDate) {
        return strtotime($formattedDate);

//        $formattedDate = explode(" ", $formattedDate);
//        if(count($formattedDate) == 1) {
//            // mm/dd/yyyy
//            return strtotime($formattedDate[0]);
//        } else {
//            //mm/dd/yyyy hh:mm:ssAM
//            $date = explode("/", $formattedDate[0]);
//            $time = explode(":", $formattedDate[1]);
//            $hour = $time[0];
//            if($hour == 12 and substr($time[2], -2) == "AM") {
//                $hour = 0;
//            } elseif($hour == 12 and substr($time[2], -2) == "PM") {
//                $hour = 12;
//            } elseif(substr($time[2], -2) == "PM") {
//                $hour = ((int)$hour + 12) % 24;
//            }
//            return mktime((int)$hour, (int)$time[1], (int)substr($time[2], 0, 2), (int)$date[0], (int)$date[1], (int)$date[2]);
//        }

    }
    public function isWIROnApplianceSupported($result) {
        $ret = false;

        if ($result == true) {
            foreach ($result as $key=>$supportedType) {
                if ($supportedType != NULL && strlen(strstr( $supportedType, "Unitrends appliance" )) > 0 ) {
                    $ret = true;
                    break;
                }
            }
        }

        return $ret;
    }

    // For Reports and other APIs that accept a date range, end_date should default to 11:59:59pm of that day
    // If a time is already specified by the user, then use that time and not 11:59:59pm of that day
    // The caller of this function should still test whether or not $end_time is false.  If it is false, then the user sent in an input that PHP strtotime does not handle
    // *******
    // ******* NOTE: This could behave differently in PHP 5.1 (CentOS5) vs PHP 5.3 (CentOS6) because strtotime changed.  Make sure to test both use cases.
    // *******
    public function formatEndDate($end_date)
    {
        // Default the end_time to being the end of the day
        $end_time = strtotime($end_date.',11:59:59pm');
        // If the end time fails being at the end of the day, then time is probably already specified for that day.
        // Thus try exactly what the user already sent in
        if ( $end_time === false )
        {
            $end_time = strtotime($end_date);
        }
        // The caller of this function should still test whether or not $end_time is false.
        return $end_time;
    }

    // Checks a given time range to see if it is valid.
    // ie. the start time is before the end time
    // Allows caller to pass whether or not future times are accepted
    // It is acceptable to pass a valid start time or end time, with the other being false or null
    public function isValidDateRange($start_time, $end_time, $allowFutureTimes = false) {
        if(is_numeric($start_time) and is_numeric($end_time)) {
            if($allowFutureTimes) {
                $isValid = $start_time <= $end_time;
            } else {
                $isValid = ($start_time <= $end_time and $end_time <= strtotime("11:59:59 pm"));
            }
        } else if(is_numeric($start_time) and !is_numeric($end_time)) {
            if($allowFutureTimes) {
                //nothing to do here
                $isValid = true;
            } else {
                $isValid = ($start_time <= time());
            }
        } else if(!is_numeric($start_time) and is_numeric($end_time)) {
            if($allowFutureTimes) {
                //nothing to do here
                $isValid = true;
            } else {
                $isValid = $end_time <= strtotime("11:59:59 pm");
            }
        } else {
            //if neither start or end time is passed in, it's technically a valid time
            $isValid = true;
        }

        return $isValid;
    }

    public function findHostByIP($ip, $sid = NULL) {
        $foundHost = NULL;
        $hosts = $this->BP->get_host_list($sid);
        if ($hosts !== false){
            foreach ($hosts as $hostName) {
                $host = $this->BP->get_host_info($hostName, $sid);
                if ($host !== false && $host['ip'] === $ip) {
                    $foundHost = $host;
                    break;
                }
            }
        }
        return $foundHost;
    }

    public function isClientWindows($os_id) {
        $is_windows = false;
        switch ($os_id) {
            case Constants::OS_WINDOWS_10:
            case Constants::OS_WINDOWS_2012_R2:
            case Constants::OS_WINDOWS_8_1:
            case Constants::OS_WINDOWS_2012:
            case Constants::OS_WINDOWS_8:
            case Constants::OS_WINDOWS_7:
            case Constants::OS_WIN_2008_R2:
            case Constants::OS_WINDOWS_2008:
            case Constants::OS_WINDOWS_VISTA:
            case Constants::OS_WINDOWS_2003:
            case Constants::OS_WINDOWS_XP:
            case Constants::OS_WINDOWS_2000:
            case Constants::OS_WINDOWS_95:
            case Constants::OS_WIN_16:
                $is_windows = true;
                break;
        }
        return $is_windows;
    }

    public function getInstanceNames($iid, $sid)
    {
        $iNames = array();
        $instanceInfo = $this->BP->get_appinst_info($iid, $sid);
        if ($instanceInfo !== false) {
            foreach ($instanceInfo as $info) {
                $appType = $info['app_type'];
                switch ($appType) {

                    case Constants::APPLICATION_TYPE_NAME_HYPER_V:
                    case Constants::APPLICATION_TYPE_NAME_EXCHANGE:
                    case Constants::APPLICATION_TYPE_NAME_FILE_LEVEL:
                    case Constants::APPLICATION_TYPE_NAME_ORACLE:
                    case Constants::APPLICATION_TYPE_NAME_SHAREPOINT:
                    case Constants::APPLICATION_TYPE_NAME_NDMP_DEVICE:

                        $iNames = array(
                            'app_type' => $appType,
                            'asset_name' => $info['primary_name'],
                            'client_name' => $info['client_name']
                        );
                        break;

                    case Constants::APPLICATION_TYPE_NAME_SQL_SERVER:
                        $iNames = array(
                            'app_type' => $appType,
                            'asset_name' => $info['secondary_name'],
                            'client_name' => $info['client_name']
                        );
                        break;

                    case Constants::APPLICATION_TYPE_NAME_VMWARE:
                        $iNames = array(
                            'app_type' => $appType,
                            'asset_name' => $info['secondary_name'],
                            'client_name' => $info['primary_name']
                        );
                        break;

                    case Constants::APPLICATION_TYPE_NAME_SYSTEM_METADATA:
                        $iNames = array(
                            'app_type' => $appType,
                            'asset_name' => $info['app_name'],
                            'client_name' => $info['client_name']
                        );
                        break;

                    case Constants::APPLICATION_TYPE_NAME_ARCHIVE:
                        $iNames = array(
                            'app_type' => $appType,
                            'asset_name' => $info['primary_name'],
                            'client_name' => $info['client_name']
                        );
                        break;

                    case Constants::APPLICATION_TYPE_NAME_XEN:
                        $iNames = array(
                            'app_type' => $appType,
                            'asset_name' => $info['secondary_name'],
                            'client_name' => $info['client_name']
                        );
                        break;
                }
                return $iNames;
            }
        }
    }

    public function isNAS($name)
    {
        return (($temp = strlen($name) - strlen(Constants::NAS_POSTFIX)) >= 0 && strpos($name, Constants::NAS_POSTFIX, $temp) !== FALSE);
    }

    public function importSets($mediaName, $sid) {
        $result = array();
        $result['storage'] = array();
        $connectedResult = $this->BP->get_connected_archive_media($sid);
        if($connectedResult !== false) {
            foreach($connectedResult as $media) {
                if(strcmp($media['name'], $mediaName) == 0 ) {
                    $mediaSets = $this->BP->get_media_archive_sets($mediaName, "", $sid);
                    if($mediaSets !== false) {
                        $numSets = 0;
                        foreach($mediaSets as $set) {
                            if($set['needs_import'] === true) {
                                $numSets++;
                            }
                        }
                        $result['storage']['sets_existing'] = count($mediaSets);
                        $result['storage']['sets_needing_import'] = $numSets;
                        if($numSets > 0) {
                            $importResult = $this->BP->import_archive_catalog($mediaName, "", false, $sid);
                            if($importResult !== false) {
                                $setsImported = count($importResult['ids']);
                                $result['storage']['sets_imported'] = $setsImported;
                                if($numSets > $setsImported) {
                                    $result['storage']['messages'] = $importResult['messages'];
                                }
                            }
                        }
                    }
                    break;
                }
            }
        } else {
            $result['storage']['messages'] = array("Failed to import sets: " . $this->BP->getError());
//                 print_r($this->BP->getError());
        }
        return $result;
    }
}
