<?php

$dir = '/api/includes/reports';

class LoadHistoryReport
{
    private $BP;

    public function __construct( $BP, $functionsClass )
    {
        $this->BP = $BP;
        $this->functions = $functionsClass;
    }

    public function get_system_load_history($systems, $data)
    {
        $report = array();
        $report_count = 0;
        //Temporary until more data is available
        $report_total = $report_count;
        $report_start = 1;
        $report_data = array();

        $start_time = false;
        if (array_key_exists('start_date', $data))
        {
            $start_time = strtotime($data['start_date']);
        }
        $end_time = false;
        if (array_key_exists('end_date', $data))
        {
            $end_time = $this->functions->formatEndDate($data['end_date']);
        }

        $isValidDate = $this->functions->isValidDateRange($start_time, $end_time, false);

        if($isValidDate) {
            foreach ($systems as $system_id => $system_name)
            {
                $load_history_result_format = array();
                $load_history_result_format['system_id'] = $system_id;

                if ($data !== NULL && count($data) > 0)
                {
                    if ($start_time !== false)
                    {
                        $load_history_result_format['start_time'] = $start_time;
                    }
                    if ($end_time !== false)
                    {
                        $load_history_result_format['end_time'] = $end_time;
                    }
                    if ( array_key_exists('order', $data) )
                    {
                        $load_history_result_format['order'] = $data['order'];
                    }
                    if ( array_key_exists('format', $data) && strtolower($data['format']) === 'xml' )
                    {
                        $load_history_result_format['format'] = $data['format'];
                    }
                    if ( array_key_exists('start_row', $data) )
                    {
                        $load_history_result_format['start_row'] = (int)$data['start_row'];
                    }
                    if ( array_key_exists('count', $data) )
                    {
                        $load_history_result_format['count'] = (int)$data['count'];
                    }
                }

                $load_history = $this->BP->get_load_history($load_history_result_format);

                if($load_history !== false)
                {
                    $loadXML = '<root>' . $load_history . '</root>';
                    $xml = simplexml_load_string($loadXML);
                    $json = json_encode($xml);
                    $loadArray = json_decode($json, TRUE);

                    if (array_key_exists("data", $loadArray))
                    {
                        $rowArray = $loadArray['data']['row'];
                        foreach ($rowArray as $loadData)
                        {
                            $temp_data_array = array();
                            $temp_data_array['day'] = date(Constants::DATE_FORMAT_US, $loadData['t']);
                            $temp_data_array['load'] = floatval($loadData['v']);
                            $temp_data_array['time'] = date(Constants::TIME_FORMAT_12H, $loadData['t']);
                            $temp_data_array['system_name'] = $system_name;

                            $report_data[] = $temp_data_array;
                            $report_count += 1;
                        }
                    }
                    if (array_key_exists("config", $loadArray))
                    {
                        $dataArray = $loadArray['config']['data']['v'];
                        $report['overload_threshold'] = floatval($dataArray[1]);
                        $report['warning_threshold'] = floatval($dataArray[0]);
                    }
                }
                else
                {
                    $report = false;
                }

                // The load history report can only be shown in the UI one system at a time.  Thus if the user passes in more than one system, only show the first one.
                break;
            }
            $report_total = $report_count;  //Temporary

            $report['count'] = $report_count;
            $report['total'] = $report_total;
            $report['start'] = $report_start;
            $report['data'] = $report_data;
        } else {
            $report['error'] = 500;
            $report['message'] = "Invalid date range provided";
        }
        return $report;
    }

}