<?php

$dir = '/api/includes/reports';

class SystemCapacityReport
{
    private $BP;

    public function __construct( $BP, $functionsClass )
    {
        $this->BP = $BP;
        $this->functions = $functionsClass;
    }

    public function get_system_capacity_report($systems)
    {
        $report = array();
        $report_count = 0;
        //Temporary until more data is available
        $report_total = $report_count;
        $report_start = 1;
        $report_data = array();

        foreach ($systems as $systemID => $systemName)
        {
            $retentionInfo = $this->BP->get_retention_info($systemID);
            if ($retentionInfo !== false and count($retentionInfo) > 0 )
            {
                $temp_system_report = array();
                $temp_system_report['system_id'] = $systemID;
                $temp_system_report['system_name'] = $systemName;
                $temp_system_report['used'] = $retentionInfo['used'];
                $temp_system_report['available'] = $retentionInfo['available'];
                $temp_system_report['message'] = ( array_key_exists('message', $retentionInfo) ? $retentionInfo['message'] : "" );
                $temp_system_report['clients'] = array();
                $temp_system_report['vms'] = array();

                if ( array_key_exists('clients', $retentionInfo) and count($retentionInfo['clients']) > 0 )
                {
                    foreach ( $retentionInfo['clients'] as $client )
                    {
                        if ( $client['name'] !== Constants::CLIENT_NAME_VCENTER_RRC )
                        {
                            $isSystemClient = false;
                            if ( $client['name'] === $systemName )
                            {
                                $tempClientID = $this->BP->get_client_id( $client['name'], $systemID );
                                $isSystemClient = ( $tempClientID === 1 );
                            }
                            if ( $isSystemClient === false )
                            {
                                $temp_client_array = array();
                                $temp_client_array['name'] = $client['name'];
                                $temp_client_array['last_full'] = $client['last_master'];
                                $temp_client_array['num_fulls'] = $client['num_masters'];

                                //apps not returned in the first out
                                $temp_client_array['apps'] = array();
                                /* Not needed for first out
                                if (array_key_exists('exchange', $client))
                                {
                                    $exchange_array = array();
                                    $exchange_array['name'] = "exchange";
                                    $exchangeInfo = $client['exchange'];
                                    foreach ($exchangeInfo as $exchange)
                                    {
                                        $temp_exchange_array = array();
                                        $temp_exchange_array['name'] = array_key_exists('database_name', $exchange) ? $exchange['database_name'] : UNKNOWN_ELEMENT;
                                        $temp_exchange_array['last_full'] = $exchange['last_full'];
                                        $exchange_array['data'][] = $temp_exchange_array;
                                    }
                                }
                                if (array_key_exists('sql', $client))
                                {
                                    $sql_array = array();
                                    $sql_array['name'] = "sql";
                                    $sqlInfo = $client['sql'];
                                    foreach ($sqlInfo as $sql)
                                    {
                                        $temp_sql_array = array();
                                        $temp_sql_array['instance_name'] = array_key_exists('instance_name', $sql) ? $sql['instance_name'] : UNKNOWN_ELEMENT;
                                        $temp_sql_array['name'] = array_key_exists('database_name', $sql) ? $sql['database_name'] : UNKNOWN_ELEMENT;
                                        $temp_sql_array['last_full'] = $sql['last_full'];
                                        $sql_array['data'][] = $temp_sql_array;
                                    }
                                }
                                if (array_key_exists('sharepoint', $client))
                                {
                                    $sharepoint_array = array();
                                    $sharepoint_array['name'] = "sharepoint";
                                    $sharepointInfo = $client['sharepoint'];
                                    foreach ($sharepointInfo as $sharepoint)
                                    {
                                        $temp_sharepoint_array = array();
                                        $temp_sharepoint_array['instance_name'] = array_key_exists('instance_name', $sharepoint) ? $sharepoint['instance_name'] : UNKNOWN_ELEMENT;
                                        $temp_sharepoint_array['name'] = array_key_exists('database_name', $sharepoint) ? $sharepoint['database_name'] : UNKNOWN_ELEMENT;
                                        $temp_sharepoint_array['last_full'] = $sharepoint['last_full'];
                                        $sharepoint_array['data'][] = $temp_sharepoint_array;
                                    }
                                }
                                if (array_key_exists('oracle', $client))
                                {
                                    $oracle_array = array();
                                    $oracle_array['name'] = "oracle";
                                    $oracleInfo = $client['oracle'];
                                    foreach ($oracleInfo as $oracle)
                                    {
                                        $temp_oracle_array = array();
                                        $temp_oracle_array['instance_name'] = array_key_exists('instance_name', $oracle) ? $oracle['instance_name'] : UNKNOWN_ELEMENT;
                                        $temp_oracle_array['name'] = array_key_exists('database_name', $oracle) ? $oracle['database_name'] : UNKNOWN_ELEMENT;
                                        $temp_oracle_array['last_full'] = $oracle['last_full'];
                                        $oracle_array['data'][] = $temp_oracle_array;
                                    }
                                }
                                if (array_key_exists('ucs', $client))
                                {
                                    $ucs_array = array();
                                    $ucs_array['name'] = "ucs";
                                    $ucsInfo = $client['ucs'];
                                    foreach ($ucsInfo as $ucs)
                                    {
                                        $temp_ucs_array = array();
                                        $temp_ucs_array['instance_name'] = array_key_exists('instance_name', $ucs) ? $ucs['instance_name'] : UNKNOWN_ELEMENT;
                                        $temp_ucs_array['name'] = array_key_exists('database_name', $ucs) ? $ucs['database_name'] : UNKNOWN_ELEMENT;
                                        $temp_ucs_array['last_full'] = $ucs['last_full'];
                                        $ucs_array['data'][] = $temp_ucs_array;
                                    }
                                }
                                */

                                $temp_system_report['clients'][] = $temp_client_array;
                                $report_count += 1;
                            }
                        }

                        if( array_key_exists('hyperv', $client) and count($client['hyperv']) > 0 )
                        {
                            foreach ($client['hyperv'] as $hyperv)
                            {
                                $temp_system_report['vms'][] = array( 'hypervisor' => "Hyper-V",
                                    'host_name' => $client['name'],
                                    'name' => $hyperv['vm_name'],
                                    'last_full' => $hyperv['last_full'],
                                    'num_fulls' => -1 );
                                $report_count += 1;
                            }
                        }

                        if( array_key_exists('vmware', $client) and count($client['vmware']) > 0 )
                        {
                            foreach ($client['vmware'] as $vmware)
                            {
                                $temp_system_report['vms'][] = array( 'hypervisor' => "VMware",
                                    'host_name' => $vmware['esx_name'],
                                    'name' => $vmware['vm_name'],
                                    'last_full' => $vmware['last_full'],
                                    'num_fulls' => -1 );
                                $report_count += 1;
                            }
                        }
                    }
                }
                $report_data[] = $temp_system_report;
            }
        }
        $report_total = $report_count;  //Temporary

        $report['count'] = $report_count;
        $report['total'] = $report_total;
        $report['start'] = $report_start;
        $report['data'] = $report_data;
        return $report;
    }

}