<?php

class SNMPConfig
{
    private $BP;

    public function __construct($BP)
    {
        $this->BP = $BP;
        $this->functions = new Functions($this->BP);
    }

    public function get_traps($oid, $sid)
    {
        global $Log;
        $results = array();

        if ($sid !== false){
            $name = $this->functions->getSystemNameFromID($sid);
            $traps = $this->BP->get_snmp_history($sid);
            if ($traps !== false) {
                $results["data"] = $this->processTraps($traps, $name, $sid, $oid);
            } else {
                $Log->writeError($this->BP->getError(), true);
                $results = false;
            }
        } else {
            $systems = $this->functions->selectSystems();
            $allTraps = array();
            foreach ($systems as $sid => $name) {
                $traps = $this->BP->get_snmp_history($sid);
                if ($traps !== false) {
                    $allTraps = array_merge($allTraps, $traps);
                } else {
                    $Log->writeError($this->BP->getError(), true);
                }
            }
            $results["data"] = $this->processTraps($allTraps, $name, $sid, -1);
        }
        return $results;
    }

    private function processTraps($traps, $system_name, $system_id, $trapID)
    {
        $allTraps = array();
        foreach ($traps as $trap) {
            if ($trapID != -1 && $trap['oid'] != $trapID) {
                continue;
            }
            $trap['system_id'] = $system_id;
            $trap['system_name'] = $system_name;
            $allTraps[] = $trap;
        }

        return($allTraps);
    }

    public function get_config($which, $sid)
    {
        $id = (int)$which;
        if ($sid == false) {
            $sid = $this->BP->get_local_system_id();
        }
        $results = array();
        $config = $this->BP->get_snmp_config($sid);
        if ($config !== false) {
            $snmp = array();
            foreach ($config as $dest) {
                if ($id != -1 && $dest['id'] !== $id) {
                    continue;
                }
                $snmp[] = $dest;
            }
            $results = array("data" => $snmp);
        } else {
            $results = false;
        }
        return $results;
    }

    public function send_test_trap($sid)
    {
        return $this->BP->send_test_snmptrap($sid);
    }

    public function put_snmp_config($config, $sid)
    {
        return $this->BP->save_snmp_config($config, $sid);
    }

    public function post_snmp_config($config, $sid)
    {
        return $this->BP->save_snmp_config($config, $sid);
    }

    public function delete_snmp_config($id, $sid)
    {
        return $this->BP->delete_snmp_config($id, $sid);
    }
}

?>
