<?php
// defines BP, xml, action, cookie
require 'header.php';

// defines functions for backup status processing.
require('basebackupstatus.php');

switch ($action) {
	case "list":	
			$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
			$cid = isset($_GET['cid']) ? (int)$_GET['cid'] : -1;
			$days = isset($_GET['days']) ? (int)$_GET['days'] : 7;
								
			//
			// Loop through all systems.  Will be only one in the list for a DPU, multiple systems on a Vault or manager.
			//
			$systems = $BP->get_system_list();
			if ($systems !== false) {
			
				$localID = $BP->get_local_system_id();
				$bVault = $BP->local_system_is_vault();	
				$bMgr = $BP->local_system_is_mgr();
				
				$crossVaultDpuID = -1;
				if ($bVault) {
					$crossVaultDpuID = getCrossVaultDpuID($BP, $systems[$localID], $systems);
				}
		
				$xml->push('BackupInformation');
				foreach ($systems as $systemID => $systemName) {

					if ($systemID == $dpuID || $dpuID == $localID) {					
						// If this system is a vault and we are looking at this system, skip it.
						if ($bVault && ($systemID == $localID)) {
							continue;
						}

						// If this system is a manager, only look at the specified dpuID.
						if (!$bVault && $bMgr && ($systemID != $dpuID)) {
							continue;
						}

						// If we cannot get the list of clients, skip it.
						$clients = $BP->get_client_list($systemID);
						if ($clients === false) {
							//createError($xml, $systemID, $systemName);
							continue;
						}						
						
						$isLocalSystem = false;
						// if the id is the same as the local system, it's the local.
						if ($systemID == $localID || $systemID == $crossVaultDpuID) {
							$isLocalSystem = true;
						}
						
						// Skip replication systems
						// need to check whether it's replicating elsewhere
						$status = getReplicationAndVaultingStatus($systemID, $isLocalSystem);
						if ($status['replicating'] === true) {
							continue;
						}
						
						// Get the backlog.
						$backlog = $BP->get_backlog($systemID);
						if ($backlog === false) {
							continue;
						}
						
						// get the securesync config.  will be needed later for SQL and baremetal backups.
						$securesync = $BP->get_securesync_config($systemID);
						
						foreach ($backlog as $backupStatus) {
							$clientID = isset($backupStatus['client_id']) ? $backupStatus['client_id'] : -1;
							$clientName = isset($backupStatus['cname']) ? $backupStatus['cname'] : "unknown";
							$backupType = isset($backupStatus['type']) ? $backupStatus['type'] : "unknown";
							
							// Skip the DPU's client file-based backups.  These will never be vaulted.
							if (($clientName == $systemName || ($clientName . '.dpu') == $systemName) && isFileBasedBackup($backupType)) {
									continue;
							}
							
							// If we are not sync'ing SQL's and this is a SQL backup type, skip it.
							// As of Release 6.1, we should no longer do this, as the securesync variable may not be true yet the SQL databases are vaulting.
							//if (skipSQL($securesync, $backupStatus)) {
							//	continue;
							//}
							
							// If we are not sync'ing baremetals and this is a baremetal type, skip it.
							if (skipBareMetal($securesync, $backupStatus)) {
								continue;
							}

							// add 'complete'.  Common XMl routine requires it and by definition they are complete.
							$backupStatus['complete'] = true;
							
							createBaseBackupStatusXML($xml, $backupStatus, NULL,
													$systemID, $systemName, $clientID, $clientName);
						}					
					}
				}
				$xml->pop(); // BackupInformation
			} else {
				$BP->buildResult($xml, false);
			}	
			echo($xml->getXml());
			break;	

}

//
// See if we are sync'ing SQLs.
//
function skipSQL($securesync, $backupStatus) {
	$bSkip = false;
	$backupType = $backupStatus['type'];
	if ($securesync !== false) {
		if ($securesync['SyncSQLBackups'] === false &&
			($backupType == "mssql full" ||
			 $backupType == "mssql differential" ||
			 $backupType == "mssql transaction")) {
			 $bSkip = true;
		}
	}
	return $bSkip;
}

//
// See if we are sync'ing BareMetals.
//
function skipBareMetal($securesync, $backupStatus) {
	$bSkip = false;
	$backupType = $backupStatus['type'];
	if ($securesync !== false) {
		if ($securesync['SyncBareMetals'] === false &&
			 $backupType == "baremetal") {
			 $bSkip = true;
		}
	}
	return $bSkip;
}

//
// Build Error.
function createError($xml, $systemID, $systemName) {
	$backup = array('size' => 'N/A',
				    'type' => 'Load Error',
					'start_time' => 0,
					'elapsed_time' => '',
					'complete' => 'N/A',
					'date' => '',
					'time' => '');
	createBaseBackupStatusXML($xml, $backup, NULL, $systemID, $systemName, "-1", "Load Error");
}


$BP->destroyCookie();
?>