<?php
//
// Define sync status bit-fields.
//
// 1 = sync needed, 2 = sync completed, 4 = sync in progress, 8 = sync wait
//
define("SYNC_NEEDED", 		0x0001);
define("SYNC_COMPLETE", 	0x0002);
define("SYNC_IN_PROGRESS",	0x0004);
define("SYNC_WAITING", 		0x0008);

//
// This function builds an expanded status which is a combination of values returned by
// bp_get_backup_status and bp_get_backup_info.
//
function createBaseBackupStatusXML($xml, $backup, $deviceList, $dpuID, $dpuName, $clientID, $clientName, $Tag = "Backup")
{
	// convert timestamp to local time/day
	$day = date('m/d/Y', $backup['start_time']);
	$time = date(TIME_FORMAT_12HR, $backup['start_time']);
		
	$backupType	= getBackupTypeString($backup['type']);

	//global $action;		
	//$Tag = ($action == 'sync') ? "SecureSyncStatus" : "Backup";
	if (!isset($backup['id'])) {
		$backup['id'] = "N/A";
	}
	$xml->push($Tag, array('Type' => $backupType, "Identification" => $backup['id'], "DPUName" => $dpuName, "ClientName" => $clientName));
	$xml->element('SystemID', $dpuID);
	$xml->element('Date', $day);
	$xml->element('Time', $time);
	$xml->element('ElapsedTime', $backup['elapsed_time']);
	$xml->element('Complete', $backup['complete'] == true ? 1 : 0);
	
	if ($backup['complete']) {
		// If the backup has completed, note this.
		if (array_key_exists('status', $backup)) {
			switch ($backup['status']) {
				case STATUS_SUCCESS:		$backupStatus = "Success"; break;
				case STATUS_WARNING:		$backupStatus = "Warning"; break;
				case STATUS_FAILURE:		$backupStatus = "Failure"; break;
			}
			$xml->element('Result', $backupStatus);
		}
	} else {
		$xml->element('Result', "Active");
	}

	$xml->element('Size', $backup['size']);
	
	// Output sync size
	if (array_key_exists('sync_size', $backup)) {
		$xml->element('SyncSize', $backup['sync_size']);
	}
	
	// If the backup has file count, note this.
	if (array_key_exists('files', $backup)) {
		$xml->element('Files', $backup['files']);
	}

	// If the backup was encrypted, note this.
	if (array_key_exists('encrypted', $backup)) {
		$xml->element('Encrypted', $backup['encrypted'] == true ? 1 : 0);
	}
	
	// Output sync status
	if (array_key_exists('sync_status', $backup)) {
		$backupStatus = createSyncStatusString($backup['sync_status']);
		$xml->element('SyncStatus', $backupStatus);
	}
	
	// Extras for operations
	if (array_key_exists('syncing', $backup)) {
		$xml->element('Syncing', "1");
	}
	if (array_key_exists('done', $backup)) {
		$xml->element('Done', $backup['done']);
	}
	
	// Extras for sync_messages
	if (array_key_exists('sync_message', $backup)) {
		$xml->push("SyncMessage");
		$xml->cData($backup['sync_message']);	
		$xml->pop();
	}
		
	if (array_key_exists('command', $backup)) {
		$xml->element('Command', $backup['command']);
	}
	
	if (array_key_exists('output', $backup)) {
		$xml->push("Output");
		$xml->cData($backup['output']);	
		$xml->pop(); // Output
	}
	
	if (array_key_exists('database_name', $backup)) {
		$xml->element('Database', $backup['database_name']);
	} else if (isset($backup['oracle_sid'])) {
		$xml->element('Database', $backup['oracle_sid']);
	}
	if (array_key_exists('server_instance_name', $backup)) {
		$xml->element('Instance', $backup['server_instance_name']);
	} else if (isset($backup['sharepoint_guid'])) {
		$xml->element('Instance', $backup['sharepoint_guid']);
	}
	// If an application, get instance ID.
	if (array_key_exists('instance_id', $backup)) {
		$xml->element('InstanceID', $backup['instance_id']);
	}
	// If an application, get name and ID.
	if (array_key_exists('app_name', $backup)) {
		$xml->emptyelement('Application', array('Name' => $backup['app_name'])); //, 'ID' => $backup['app_id']));
	}
	
	// replication specifics.
	if (isset($backup['Rep_Instance'])) {
		$xml->element('TargetName', $backup['Rep_Target']);
		$xml->element('InstanceName', $backup['Rep_Instance']);
	}
	
	// If VMware Template
	if (isset($backup['vmware_template'])) {
		$xml->element('Template', ( (bool)$backup['vmware_template'] == TRUE ) ? 1 : 0 );
	}
	
	// VSS Support
	if (isset($backup['support_vss_diff'])) {
		$xml->element('VSSDiff', ( (bool)$backup['support_vss_diff'] == TRUE ) ? 1 : 0 );
	}

	// If Certified
	if (isset($backup[STATUS_CERTIFY_API_KEY])) {
		$certifyAttributes = getCertifyStatusAttributesArray($backup[STATUS_CERTIFY_API_KEY]);
		$xml->emptyelement(STATUS_CERTIFY_XML_NODE, $certifyAttributes);
	}
		
	$xml->pop(); // Backup
}

//
// This function gets the URL arguments and pulls out the start and end times, if set,
// and adds them to an attributes array, which is returned.
// getFinishTimeInterval - if true, this function will return the finish_interval_start and
// finish_interval_end attributes. These are used by bp_get_backup_status API
// when you want to retrieve backups that finished during this time interval (rather than started)
//
function getTimeArguments($getFinishTimeInterval=false)
{
	$attributes = array();

	$startTime = -1;
	$endTime = -1;
	
	$options = new Options();
	
    $startTimeKey;
    $endTimeKey;
        
    if ($getFinishTimeInterval !== true) {
        $startTimeKey = 'start_time';
        $endTimeKey = 'end_time';
    }
    else {
        $startTimeKey = 'finish_interval_start';
        $endTimeKey = 'finish_interval_end';
    }
    
	if (isset($_GET['arguments'])) {
		$options->parseCommandLine($_GET['arguments']);
		$startTime = $options->getStartTime();
		if ($startTime != -1) {
			$attributes = array($startTimeKey => $startTime);
		}
		$endTime = $options->getEndTime();
		if ($endTime != -1) {
			$attributes = array_merge($attributes, array($endTimeKey => $endTime));
		}
	}
	
	return $attributes;
}

// Create and return a sync status string, given a sync status binary value.
function createSyncStatusString($status)
{
	if ($status == 0) {
		$strStatus = 'N/A';
	} else {
		$strStatus = '';
		$charSeparator = '';
		if ($status & SYNC_WAITING) {
			$strStatus .= 'Waiting';
			$charSeparator = ',';
		}
		if ($status & SYNC_IN_PROGRESS) {
			$strStatus .= $charSeparator . 'In Progress';
			$charSeparator = ',';
		}
		if ($status & SYNC_COMPLETE) {
			$strStatus .= $charSeparator . 'Complete';
			$charSeparator = ',';
		}
		if ($status & SYNC_NEEDED) {
			$strStatus .= $charSeparator . 'Needed';
		}
	}
	return $strStatus;
}


//
// Function that compares the start dates of two function,
// sort DESCENDING order.
//
function compareDates($a, $b) {
	if ($a['start_time'] == $b['start_time']) {
		$result = 0;
	} else if ($a['start_time'] > $b['start_time']) {
		$result = -1;
	} else {
		$result = 1;
	}
	return $result;
}


//
// This function translates the user-readable type to an underlying type.
//
function getTypeForDetails($bSync, $bType)
{
	$backupType = NULL;
	
	if ($bSync) {	
		switch ($bType) {
			case "Master":
				$backupType = "securesync master";
				break;
			case "Incremental":
				$backupType = "securesync incremental";
				break;
			case "Differential":
				$backupType = "securesync differential";
				break;
			case "Bare Metal":
			case "BareMetal":
				$backupType = "securesync baremetal";
				break;
			case "Local Directory":
				$backupType = "securesync localdir";
				break;
			case "DPUstate":
			case "SystemState";
				$backupType = "securesync dpustate";
				break;
			case "Exchange Full":
				$backupType = "securesync msexch full";
				break;
			case "Exchange Differential":
				$backupType = "securesync msexch diff";
				break;
			case "Exchange Incremental":
				$backupType = "securesync msexch incr";
				break;
			case "SQL Full":
				$backupType = "securesync mssql full";
				break;
			case "SQL Differential":
				$backupType = "securesync mssql diff";
				break;
			case "SQL Transaction":
				$backupType = "securesync mssql trans";
				break;
			case "VMware Full":
				$backupType = "securesync vmware full";
				break;
			case "VMware Differential":
				$backupType = "securesync vmware diff";
				break;
			case "VMware Incremental":
				$backupType = "securesync vmware incr";
				break;
			case "Exchange":
				$backupType = "securesync exchange";
				break;
			case "SQL":
				$backupType = "securesync sql";
				break;
			case "HyperV Full":
				$backupType = "securesync hyperv full";
				break;
			case "HyperV Incremental":
				$backupType = "securesync hyperv incr";
				break;		
			/*case "Block Full":
				$backupType = "securesync block full";
				break;
			case "Block Incremental":
				$backupType = "securesync block incr";
				break;			*/			
		}
	} else {
		switch ($bType) {
			case "Master":
				$backupType = "master";
				break;
			case "Incremental":
				$backupType = "incremental";
				break;
			case "Differential":
				$backupType = "differential";
				break;
			/*case "Block Full":
				$backupType = "block full";
				break;	
			case "Block Incremental":
				$backupType = "block incremental";
				break;	*/
			case "Bare Metal":
			case "BareMetal":
				$backupType = "baremetal";
				break;
		}
	}
	
	return $backupType;
}

//
// This function builds an expanded status which is a combination of values returned by
// bp_get_backup_status and bp_get_backup_info.
//
function createShortStatusXML($xml, $backup)
{
	
	$epoch = $backup['start_time'];
		
	$backupType	= getBackupTypeString($backup['type']);

	$xml->push('Backup', array('Type' => $backupType, "ID" => $backup['id']));
	$xml->element('Start', $epoch);
	$xml->element('ElapsedTime', $backup['elapsed_time']);
	$xml->element('Complete', $backup['complete'] == true ? 1 : 0);
	
	if ($backup['complete']) {
		// If the backup has completed, note this.
		if (array_key_exists('status', $backup)) {
			switch ($backup['status']) {
				case STATUS_SUCCESS:		$backupStatus = "Success"; break;
				case STATUS_WARNING:		$backupStatus = "Warning"; break;
				case STATUS_FAILURE:		$backupStatus = "Failure"; break;
			}
			$xml->element('Result', $backupStatus);
		}
	} else {
		$xml->element('Result', "Active");
	}

	// If an application, get instance ID.
	if (array_key_exists('instance_id', $backup)) {
		$xml->element('InstanceID', $backup['instance_id']);
	}
	// If an application, get name and ID.
	if (array_key_exists('app_name', $backup)) {
		$xml->emptyelement('Application', array('Name' => $backup['app_name'])); //, 'ID' => $backup['app_id']));
	}
		
	$xml->pop(); // Backup
}


//
// This function builds an expanded status which is a combination of values returned by
// bp_get_backup_status and bp_get_backup_info.
//
function createBackupExpandedStatusXML(&$xml, &$backup, &$deviceList, &$clientList, $dpuID, $dpuName)
{
	global $BP;
	// convert timestamp to local time/day
	$day = date('m/d/Y', $backup['start_time']);
	$time = date(TIME_FORMAT_12HR, $backup['start_time']);
	$endTime = date(TIME_FORMAT_12HR, $backup['start_time'] + $backup['elapsed_time']);
		
	$backupType	= getBackupTypeString($backup['type']);
	
	// See if the restored backup type is set.  If so, this is the type we want to display.
	// Currently only applies to type 'vrestore' when using bp_get_baciup_status.
	if (isset($backup['restored_bkup_type']) && $backup['type'] == "virtual restore") {
		$backupType	= getBackupTypeString($backup['restored_bkup_type']);
	}
					
	$clientName = clientNameOf($backup['client_id'], $clientList);

	if ($clientName == "" || $clientName == NULL) {
		//see if it's a v-client
		$vclients = $BP->get_virtual_client_list($dpuID);
		foreach ($vclients as $vclient) {
			if (isset($vclient['real_client_id'])) {
				if ($vclient['real_client_id'] == $backup['client_id']) {
					$clientName = $vclient['client_name'];
					$dpuName = $vclient['system_name'];
				}
			}
		}
	}
	
	$attributes = array('Type' => $backupType,
						"DPUName" => $dpuName,
						"ClientName" => $clientName);
						
	if (isset($backup['id'])) {
		$attributes['Identification'] = $backup['id'];
	}
	$xml->push("Backup", $attributes);
	$xml->element('Date', $day);

	$xml->element('Time', $time);
	$xml->element('ElapsedTime', $backup['elapsed_time']);
	$xml->element('EndTime', $endTime);
	$xml->element('Complete', $backup['complete'] == true ? 1 : 0);
	
	if (isset($backup['last'])) {
		$xml->element('Last', $backup['last'] == true ? 1 : 0);
	}
	
	if (isset($backup['complete'])) {
		if (isset($backup['complete']) && $backup['complete'] == true) {
        // If the backup has completed, note this.
			if (isset($backup['status'])) {
				$backupStatus = getBackupStatusString($backup);
				$xml->element('Result', $backupStatus);
			}
		
    	} else {
        	$xml->element('Result', "Active");
    	}
	}


	// If the backup was verified, note this.
	if (array_key_exists('verify_status', $backup) && $backup['type'] != 'verify') {
		$xml->element('VerifyResult', $backup['verify_status']);
	}

	$xml->element('Size', $backup['size']);
	
	if(isset($backup['sync_size'])){
		$xml->element('SyncSize', $backup['sync_size']);
	}
	// If the backup has file count, note this.
	// if type is 'verify', show "N/A" rather than 0
	if (isset($backup['files'])) {
		if($backup['type'] == 'verify'){
			$xml->element('Files', "N/A");
		}else{
			$xml->element('Files', $backup['files']);
		}
	}
	
	// If the backup was encrypted, note this.
	if (isset($backup['encrypted'])) {
		$xml->element('Encrypted', $backup['encrypted'] == true ? 1 : 0);
	}
	
	// Output sync status
	if (isset($backup['sync_status'])) {
		$backupStatus = createSyncStatusString($backup['sync_status']);
		$xml->element('SyncStatus', $backupStatus);
	}
	
	// If an MSSQL, Exchange, or Oracle get instance and database names.
	if (isset($backup['server_instance_name'])) {
		$xml->element('Instance', $backup['server_instance_name']);
	} else if (isset($backup['sharepoint_guid'])) {
		$xml->element('Instance', $backup['sharepoint_guid']);
	}
	if (isset($backup['database_name'])) {
		$xml->element('Database', $backup['database_name']);
	} else if (isset($backup['oracle_sid'])) {
		$xml->element('Database', $backup['oracle_sid']);
	} else if (isset($backup['vm_name'])) {
		// return vm_name in the database field for replication history report
		$xml->element('Database', $backup['vm_name']);
	}
	
	// If an application, get instance ID.
	if (array_key_exists('instance_id', $backup)) {
		$xml->element('InstanceID', $backup['instance_id']);
	}
	
	// If an application, get name and ID.
	if (array_key_exists('app_name', $backup)) {
		$xml->emptyelement('Application', array('Name' => $backup['app_name'])); //, 'ID' => $backup['app_id']));
	}
	

	
	// device specific information
	if (isset($backup['compressed'])) {
		$xml->element('Compressed', $backup['compressed'] == true ? 1 : 0);
	}
	
	// device specific information
	if (isset($backup['filename'])) {
		$xml->element("Filename", $backup['filename']);
	}
	
	if (isset($backup['device_id'])) {
		$attributes = array('ID' => $backup['device_id']);
		if ($deviceList !== false) {
			$attributes = array_merge($attributes, array('Name' => deviceNameOf($backup['device_id'], $deviceList)));
		}
		$xml->emptyelement('Device', $attributes);
	}
	
	if (isset($backup['command'])) {
		$xml->element('Command', $backup['command']);
	}
	
	// Extras for sync_messages
	if (isset($backup['sync_message'])) {
		$xml->push("SyncMessage");
		$xml->cData($backup['sync_message']);	
		$xml->pop();
	}
	
	if (isset($backup['output'])) {
		$xml->push("Output");
		$xml->cData($backup['output']);	
		$xml->pop(); // Output
	}
	
	if (isset($backup['comment'])) {
		$xml->element('Comment', $backup['comment']);
	}

	if (isset($backup['purgeable'])) {
		$xml->element('Purgeable', $backup['purgeable'] == true ? 1 : 0);
	}
	
	if (isset($backup['support_vss_diff'])) {
		$xml->element('VSSDiff', $backup['support_vss_diff'] == true ? 1 : 0);
	}
	
	// If an MSSQL Type, get group number and order.
	if (isset($backup['group_number'])) {
		$xml->element('GroupNumber', $backup['group_number']);
	}
	if (isset($backup['group_order'])) {
		$xml->element('GroupOrder', $backup['group_order']);
	}

	// see if synthesized or synthesis capable.
	$xml->element('Synth', (isset($backup['synthesized']) && $backup['synthesized'] == true) ? 1 : 0);	
	//$xml->element('SynthCapable', (isset($backup['synth_capable']) && $backup['synth_capable'] == true) ? 1 : 0);
	
	// See if disk metadata file is present.
	if (isset($backup['disk_metadata'])) {
		$xml->element('Meta', $backup['disk_metadata'] == true ? 1 : 0);
	}
	
		// See if client config is present - used to get config from backup
	if (isset($backup['client_config'])) {
		$xml->element('ClientConfig', $backup['client_config'] == true ? 1 : 0);
	}
	
	//if hyper-v, is it a cluster?
	if (isset($backup['is_cluster'])) {
		$xml->element('Cluster', $backup['is_cluster'] == true ? 1 : 0);
	}
	
	//for hyper-v, get guid
	if (isset($backup['guid'])) {
		$xml->element('guid', $backup['guid']);
	}
	//for hyper-v restore report, get VM name
	if (isset($backup['vm_name'])) {
		$xml->element('VM', $backup['vm_name']);
	}
	
	// see if backup is replicated.
	if (isset($backup['replicated']) && $backup['replicated'] == true) {
		$xml->element('Replicated', 1);
	}
	
	// If VMware Template
	if (isset($backup['vmware_template'])) {
		$xml->element('Template', ( (bool)$backup['vmware_template'] == TRUE ) ? 1 : 0 );
	}
	
	// VSS Support
	if (isset($backup['support_vss_diff'])) {
		$xml->element('VSSDiff', ( (bool)$backup['support_vss_diff'] == TRUE ) ? 1 : 0 );
	}
	
	// bug 16757, LD - Legal Hold changes
	// see if backup is on legal hold.
	if (isset($backup['is_on_hold'])) {
		$xml->push('LegalHold');
		if (isset($backup["is_on_hold"])) {
			$xml->element("OnHold", $backup["is_on_hold"] == true ? "1" : "0");
		}
		if (isset($backup["hold_days_backup"])) {
			$xml->element("LegalHoldDaysBackup", $backup["hold_days_backup"]);
		}
		if (isset($backup["hold_days_instance"])) {
			$xml->element("LegalHoldDaysInstance", $backup["hold_days_instance"]);
		}
		if (isset($backup['hold_expire_time'])) {
			// convert timestamp to local time/day
			$day = date('m/d/Y', $backup['hold_expire_time']);
			$time = date(TIME_FORMAT_12HR, $backup['hold_expire_time']);
			$xml->element('LegalHoldExpirationDate', $day);
			$xml->element('LegalHoldExpirationTime', $time);
		}
		$xml->pop();
	}

	// If Certified
	if (isset($backup[STATUS_CERTIFY_API_KEY])) {
		$certifyAttributes = getCertifyStatusAttributesArray($backup[STATUS_CERTIFY_API_KEY]);
		$xml->emptyelement(STATUS_CERTIFY_XML_NODE, $certifyAttributes);
	}
	
	$xml->pop(); // Backup
}

//
// This function returns the name of the device in the list
// of devices with id  $id.
//
function deviceNameOf($deviceID, $devices)
{
	$deviceName = "";
	if (is_array($devices)) {
		if (array_key_exists($deviceID, $devices)) {
			$deviceName = $devices[$deviceID];
		}
	}
	return $deviceName;
}

//
// This function returns the name of the client in the list
// of client with id  $id.
//
function clientNameOf($clientID, $clients)
{
	$clientName = "";
	if (is_array($clients)) {
		if (array_key_exists($clientID, $clients)) {
			$clientName = $clients[$clientID];
		}
	}
	return $clientName;
}

?>