<?php
// defines BP, xml, action, cookie
require 'header.php';
require 'clientappsxml.php';
require 'credentials.php';

// general library functions
//require 'library.php';  not needed, required by clientappsxml.php

// set ClientInfoID ID to null (for update vs. add)
$clientInfoID = NULL;
$clientInfo = array();
$Credentials = new Credentials();

// VCenter-RRC is a special client.
define("VCENTER_RRC", "vCenter-RRC");

// Used in Displaying volumes in Client Configuration.
define("BASE_APP_STRING", " (");

$localID = $BP->get_local_system_id();
$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $localID;
$bHideApps = false;

switch ($action) {

	case "list":
		$bValid = true;
		if ($grandClientView) {
			$clients = $BP->get_grandclient_list($dpuID);
			// working on the local system.
			$dpuID = $localID;
		} else {
			$clients = $BP->get_client_list($dpuID);
		}
		if ($clients === false) {
			$bValid = false;
		} else {
			$xml->push("root");
			foreach ($clients as $id => $name) {
				$clientInfo = $BP->get_client_info($id, $dpuID);
				if ($clientInfo === false) {
					$bValid = false;
					break;
				} else {
					$networkID = sprintf("%d.%d", $dpuID, $id);
					$Credentials = NULL;
					if (isset($clientInfo['credentials']) && count($clientInfo['credentials']) > 0) {
						$Credentials = new Credentials();
					}				
					createClientXML($xml, $clientInfo, $id, $networkID, true, $Credentials, $dpuID);
				}
			}
			// add vCenters if present
			$result = $BP->get_vcenter_list($dpuID);
			if ($result !== false) {
				foreach ($result as $id => $name) {
					$xml->push('VCenter', array("ID" => $id, "Name" => $name));
					$info = $BP->get_vcenter_info($id, $dpuID);
					if ($info !== false) {
						if (isset($info['address'])) {
							$xml->element('Address', $info['address']);
						}
						if (isset($info['ip'])) {
							$xml->element('IP', $info['ip']);
						}
						if (isset($info['username'])) {
							$xml->element('Username', $info['username']);
						}
					}
					$xml->pop(); // VCenter
				}
			}
			$xml->pop(); //root
		}
		if (!$bValid) {
			$xml->clear(false);
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
					
	case "update":	
		// if updating, see if user changed show/hide app selections								
		if (isset($_REQUEST['application'])){
			$bHideApps = true;					
			$clientAppsXML = $_REQUEST['application'];  //xml passed in
			$clientApps = xmlToClientApps($clientAppsXML);
		}
		// if updating, specify id or map client name/system name to client ID				
		if (isset($_GET['id'])) {
			$id = (int)$_GET['id'];
			$clientInfoID = array("id" => $id);				
		} else {
			$BP->buildResult($xml, false, "client update must have id specified.");
			echo($xml->getXml());
			break;
		}
		// fall through to add.
	
	case "add":		
		// Get the client definition from arguments.
		if (isset($_GET['name'])) {
			$name = $_GET['name'];
			$clientInfo = array ("name" => $name);
		}
			
		//check if client should be added to default schedule
		if (isset($_GET['defaultschedule'])) {
			$addToDefault = $_GET['defaultschedule'];	
			global $Log;
			$Log->writeVariable("add to default " . $addToDefault);	
		}

		// See if we are adding or updating.  If updating, $clientInfoID ($id) will be set.
		if ($clientInfoID != NULL) {
			$clientInfo = array_merge($clientInfoID, $clientInfo);
		}
		
		// see if we need to associate w/credentials.
		if (isset($_GET['crid'])) {
			$crid = (int)$_GET['crid'];
			if ($crid == -1) {
				$clientInfo["no_credentials"] = true;
			} else {
				$clientInfo["credential_id"] = $crid;
			}
		}
		
		// If a virtual client, ensure it is in the hosts file.
		if ($name === VCENTER_RRC) {
			$lib = new Library();
			$bValid = $lib->addHostAlias($BP, 'localhost', VCENTER_RRC, $dpuID);
		} else if (isset($_GET['oid']) and (int)$_GET['oid'] == 40 and isset($_GET['genericProperty']) and (int)$_GET['genericProperty'] == 4) {
			$lib = new Library();
			$bValid = $lib->addHostAlias($BP, 'localhost', $name, $dpuID);
		}
		
											
		$options = array ('os_type_id' => 'oid', 'machine_type_id' => 'mid');
		$bOptions = array('is_enabled' => 'enabled',
						  'is_synchable' => 'sync',
						  'is_encrypted' => 'encrypt');
		$clientInfo = buildOptions($clientInfo, $options, $bOptions);
		$iOptions = array('priority' => 'priority');
		$clientInfo = addIntegerOptions($clientInfo, $iOptions);
		
		if ( isset ($_GET['oid']) && $_GET['oid'] == '40' && isset($_GET['genericProperty']) ) {
			$useRaeSave = true;
			$clientInfo = addIntegerOptions($clientInfo, array ('os' => 'oid', 'generic_property' => 'genericProperty') );
		} else {
			$useRaeSave = false;
		}

		global $Log;

		$Log->writeVariable( "Setting the value for the port" );
		if ( isset( $_GET['port'] )) {
			$useRaeSave = false;
			$clientInfo['remote_port'] = (int)$_GET['port'];
			$Log->writeVariable( "The port is ".$clientInfo['port'] );

			$Log->writeVariable( $useRaeSave ? "Saving with RAE" : "Saving old way" );
		}  else {
			$Log->writeVariable( $useRaeSave ? "Saving with RAE - no port" : "Saving old way - no port" );
		}
		
		$Log->writeVariable( "Setting value for the IP" );
		if ( isset( $_GET['ip'] )) {
			$useRAESave = false;
			$clientInfo['remote_address'] = $_GET['ip'];
			$Log->writeVariable( "The ip is ".$clientInfo['remote_address'] );
			
			$Log->writeVariable( $useRaeSave ? "Saving with RAE" : "Saving old way" );
		} else {
			$Log->writeVariable( $useRaeSave ? "Saving with RAE - no ip" : "Saving old way - no ip" );
		}
		
		// only add SSL if specified.
		if (isset($_GET['ssl'])) {
			$clientInfo['use_ssl'] = $_GET['ssl'] == '1' ? true : false;
		}
		if ($bHideApps && $name !== VCENTER_RRC){
			$clientInfo['applications'] = $clientApps;
		}

		if ($useRaeSave) {
			$result = $BP->save_rae_client_info($clientInfo, $dpuID);
		} else {
			$result = $BP->save_client_info($clientInfo, $dpuID);
		}
		if ($result !== false) {
			
			if ($clientInfoID == NULL && $name !== VCENTER_RRC) {
				// If adding a client ($clientInfoID == NULL) and not client vCenter-RRC, create a schedule.
				// Let the user knows if this step is not successful.
				if ($addToDefault){
					// Comment this out.  Do not add to default global schedule any more.  Create client schedule.
					//$result = $BP->add_client_to_default_schedule($clientInfo['name'], $dpuID);
					$result = createClientSchedule($clientInfo, $dpuID);
					if ($result === false) {
						$msg .=	"Successfully added client " . $clientInfo['name'] . ", but a backup schedule was not " .
								"automatically created: " . $BP->getError();
						$xml->lastTag('ResultArray', 'Result', array('Message' => $msg));
					}
				}
				//$msg = "To archive data from this client, please add the client to an archive schedule.";
				//$xml->lastTag('DefaultSchedule', 'Information', array('Message' => $msg));
			}
			$BP->buildResult($xml, true);
		} else {
			// on failure, return error to user.
			// At one time, we were deleting the associated credentialID, but should not as we have a credentials manager.
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());	
		break;
					
	case "delete":
		// get client name if specified.
		$clientName  = isset($_GET['name']) ? $_GET['name'] : "";
		$hasAliases  = isset($_GET['hasalias']) ? true : false;
		$removeAlias = isset($_GET['remalias']) ? true : false;
		$alias       = isset($_GET['alias']) ? $_GET['alias'] : "";
		
		// if the client is an alias, deletion of the alias will result in deletion 
		// of the client from the hosts file and the inability for the system to reach the client
		$clientHostName = isset($_GET['hostname']) ? $_GET['hostname'] : NULL;
		
		// If the client is an alias, the hostName is not the same as the alias name
		// and thus the alias should be deleted from the host file - not the main client.
		// if the hostName is not passed up, the client might not be a version capable
		// of providing the hostName contents.
		// Even though host names and aliases should be case sensitive, we use them in a case insensitive
		// fashion in other parts of the system, so I am doing a case insensitive compare.
		$clientIsAlias = $clientHostName != NULL && strcasecmp( $clientName, $clientHostName ) != 0;
		
		if (isset($_GET['id'])) {
			// client ID specified
			$clientID = (int)$_GET['id'];
		} else {
			// translate client name to ID.
			$clientID = getClientIDFromName($BP, $clientName, $dpuID);
		}
		if ($clientID != -1) {
			// a grand client is local.
			if ($grandClientView) {
				$dpuID = $localID;
			}
			$result = $BP->delete_client($clientID, $dpuID);
			if (!$grandClientView && $result !== false && $clientName != "") {
				$hostResult = false;
			
				// If successfully deleted and not a grandclient, remove from the hosts file; log, don't flag an error.
				if ($clientName == VCENTER_RRC || strstr($clientName, NAS_POSTFIX)) {
					// delete aliased host entry from localhost if vCenter or a NAS.
					$lib = new Library();
					$hostResult = $lib->deleteHostAlias($BP, 'localhost', $clientName, $dpuID);
				} else if ( $clientIsAlias && $removeAlias ) {
					$lib = new Library();
					$hostResult = $lib->deleteHostAlias($BP, $clientHostName, $alias, $dpuID);
				} else if ( $hasAliases === false ) {
					$hostResult = $BP->remove_host_info($clientName, $dpuID);
				}
				if ($hostResult == false) {
					global $Log;
					$msg = 'Cannot remove host ' . $clientName . ' ' . $BP->getError();
					$Log->writeError($msg, true);
				}
			}
			$BP->buildResult($xml, $result);
		} else {
			$stringError = sprintf("client delete: could not determine id from name %s.",
									$clientName);
			$BP->buildResult($xml, false, $stringError);
		}
		echo($xml->getXml());
		break;
		
	case "vaulting":
		// see if the client ID is vaulting, either file-based or application backups.
		$clientID = isset($_GET['id']) ? (int)$_GET['id'] : -1;
		$result = $BP->is_client_vaulting($clientID, $dpuID);
		if ($result === -1) {
			$BP->buildResult($xml, false);
		} else {
			$xml->push('root');
			$xml->element('Vaulting', $result == true ? 1 : 0);
			$xml->pop(); // root
		}
		echo($xml->getXml());
		break;
		
	case "config":
		// get the client configuration and return.
		$clientID = isset($_GET['id']) ? (int)$_GET['id'] : -1;
		$client = $BP->get_client_config($clientID, $dpuID);
		$clientInfo = $BP->get_client_info($clientID);
		
		if ($client !== false) {
			$xml->push('root');
			$xml->element('Processors', $client['processors']);
			$xml->element('Memory', $client['memory']);
			if (count($client['disks']) > 0) {
				$diskCount = (count($client['disks']));  //11/14/13 - bug 18197 
				
				$disks = $client['disks'];
				$i = 0;
				$sep = "";
				$s = "";
				foreach ($disks as $disk) {
					$s .= $sep . $disk;
					$sep = ",";
				}
				$xml->element('DiskCount', $diskCount);
				$xml->element('Disks', $s);
			}
			if (count($client['volumes']) > 0) {
				$volumes = $client['volumes'];
				$i = 0;
				foreach ($volumes as $volume) {
					// Each volume is an array with label and critical indicator.
					$volumeLabel = $volume['label'];
					$nCritical = $volume['critical'] ? 1 : 0;
					$volumeView = createVolumeView($volume);
					
					$attributes = array('Filename' => $volumeLabel,
										'id' => $i, 'Name' => $volumeLabel,
										'View' => $volumeView,
										'Critical' => $nCritical);
					$xml->emptyElement('Volume', $attributes);
					$i++;
				}
			}
			// Get the boot disk value if set (it should be); -1 if not set.
			$bootDisk = isset($client['boot_disk']) ? $client['boot_disk'] : -1;
			$xml->element('Boot', $bootDisk);
			
			// efi support
			if (isset($client['efi'])) {
				$xml->element('efi', $client['efi']);
			}
			
			$xml->pop(); // root
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
		
		case "config_from_backup":
		// get the client configuration from the backup and return.
		$backupID = isset($_GET['bid']) ? (int)$_GET['bid'] : -1;
		$client = $BP->get_client_config_from_backup($backupID, $dpuID);
				
		if ($client !== false) {
			$xml->push('root');
			$xml->element('Processors', $client['processors']);
			$xml->element('Memory', $client['memory']);
			if (count($client['disks']) > 0) {
				$diskCount = (count($client['disks']));  //11/14/13 - bug 18197 
				
				$disks = $client['disks'];
				$i = 0;
				$sep = "";
				$s = "";
				foreach ($disks as $disk) {
					$s .= $sep . $disk;
					$sep = ",";
				}
				
				$xml->element('DiskCount', $diskCount);
				$xml->element('Disks', $s);
				
			}
			if (count($client['volumes']) > 0) {
				$volumes = $client['volumes'];
				$i = 0;
				foreach ($volumes as $volume) {
					// Each volume is an array with label and critical indicator.
					$volumeLabel = $volume['label'];
					$nCritical = $volume['critical'] ? 1 : 0;
					$volumeView = createVolumeView($volume);
					
					$attributes = array('Filename' => $volumeLabel,
										'id' => $i, 'Name' => $volumeLabel,
										'View' => $volumeView,
										'Critical' => $nCritical);
					$xml->emptyElement('Volume', $attributes);
					$i++;
				}
			}
			// Get the boot disk value if set (it should be); -1 if not set.
			$bootDisk = isset($client['boot_disk']) ? $client['boot_disk'] : -1;
			$xml->element('Boot', $bootDisk);
			
			// efi support
			if (isset($client['efi'])) {
				$xml->element('efi', $client['efi']);
			}
			
			$xml->pop(); // root
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
}

//
// Given the client's name, return its ID.
//
function getClientIDFromName($BP, $clientName, $dpuID) {
	return $BP->get_client_id($clientName, $dpuID);
}

//
// Creates a volume view string that contains extra information about this volume.
//
						   
function createVolumeView($volume) {
	
	$volumeView = $volume['label'];
	// If critical, add a note to that effect.
	$volumeView .= $volume['critical'] == true ? ' (Critical)' : '';
	
	// Create an application string, based on what is installed on this volume.
	// We'll build a parenthesized list, comma-separated.
	//
	$stringApps = BASE_APP_STRING;
	$sep = "";
	// Is our Windows agent on this volume?
	if (isset($volume['bpdir']) && $volume['bpdir'] == true) {
		$stringApps .= "Agent";
		$sep = ", ";
	}
	// Is system state on this volume?
	if (isset($volume['sstate']) && $volume['sstate'] == true) {
		$stringApps .= $sep . "System State";
		$sep = ", ";
	}
	// Are any Exchnage databases or storage groups on this volume?
	if (isset($volume['exch']) && $volume['exch'] == true) {
		$stringApps .= $sep . "Exchange";
		$sep = ", ";
	}
	// Are any SQL databases on this volume?
	if (isset($volume['sql']) && $volume['sql'] == true) {
		$stringApps .= $sep . "SQL";
	}
	// If we created an apps string, add it now.
	if ($stringApps != BASE_APP_STRING) {
		$volumeView .= $stringApps . ")";
	}
				
	return $volumeView;
}

function createClientSchedule($clientInfo, $dpuID) {
	global $BP;
	
	// Use Friday Master, Daily Incremental as the default.
	$iCal = <<< EOM
BEGIN:VCALENDAR
BEGIN:VEVENT
SUMMARY:Master
DESCRIPTION:weekly master
DTSTART:20120603T030000
DTEND:20120603T040000
RRULE:FREQ=WEEKLY;BYDAY=SU
END:VEVENT
BEGIN:VEVENT
SUMMARY:Incremental
DESCRIPTION:daily incremental
DTSTART:20120604T030000
DTEND:20120604T040000
RRULE:FREQ=WEEKLY;BYDAY=MO,TU,WE,TH,FR
END:VEVENT
END:VCALENDAR
EOM;

	$clientID = $BP->get_client_id($clientInfo['name'], $dpuID);
	
	$defaultName = $clientInfo['name'] . ' Schedule';
	$description = 'Automatically created backup schedule for ' . $clientInfo['name'];
	
	$backupOptions = 	array(	'verify_level' => 3);			// set verify level to inline by default.
	$scheduleOptions = 	array(	'client_id' => $clientID,
								'include_new' => false,			// cannot be true for file-level schedule
								'email_report' => true,
								'failure_report' => true);
	
	$scheduleInfo = 	array(	'name' => $defaultName,
								'description' => $description,
								'enabled' => true,				// create as enabled.
								'app_id' => 1,					// app_id 1 is file-level
								'calendar' => $iCal,
								'backup_options' => $backupOptions,
								'schedule_options' => $scheduleOptions,
                                'regular_expressions' => array() );
	
	$result = $BP->save_app_schedule_info($scheduleInfo, $dpuID);
	
	return $result;
}

$BP->destroyCookie();
?>
