<?php

// class to wrap storage functions.
class StorageFunctions
{
	function __construct()
	{
	}
	
	//
	// Given a device list and system ID, produce XML of each device details.
	//
	function getDevicesDetails($BP, $xml, $deviceList, $dpuID) {
		foreach ($deviceList as $id => $name) {
			$deviceInfo = $BP->get_device_info($id, $dpuID);
			if ($deviceInfo === false) {
				$deviceInfo = array('dev_name', $name);
			} else {
				$networkID = sprintf("%d.%d", $dpuID, $id);
				createDeviceXML($xml, $deviceInfo, $id, $networkID);
			}
		} 
	}
	
	//
	// Computes the space used on the device using the 'du' utility.
	//
	function getDeviceSpaceUsed($BP, $xml, $deviceName, $capacity){
		$TotalBytes = -1;

		$path = "/backups/" . $deviceName . '/';
		$command = sprintf("sudo -S /usr/bin/du -s -b %s", $path);
		$handle = popen($command, 'r');
		
		if ($handle) {
			$commandReturn = fread($handle, 1024);
			pclose($handle);
		}
		
		$TotalBytes = $commandReturn;
		$deviceUsedGB = toNumberString($TotalBytes / (GIGABYTE * 1024));
		$deviceFreeGB = toNumberString(($capacity - $deviceUsedGB) ,2);
		$xml->push("space");
		$xml->element("used", $deviceUsedGB);				
		$xml->element("free", $deviceFreeGB);				
		
		$xml->pop();
	}
		
	//
	// Given a storage information array and the device list, produce storage info XML.
	//
	function createStorageXML($BP, $xml, $infoArray, $deviceList, $dpuID) {
		$storage = $infoArray['name'];
	
		$xml->push('Storage', array('Name' => $storage, 'ID' => $infoArray['id']));
		if (isset($infoArray['type'])) {
			$xml->element('Type', $infoArray['type']);
			$xml->element('TypeString', $this->getStorageType($infoArray['type']));
		}
		if (isset($infoArray['usage'])) {
            $xml->element('Usage', $infoArray['usage']);
        }
        if (isset($infoArray['stateless_type'])) {
            $xml->element('StatelessType', $infoArray['stateless_type']);
        }
		//
		// Get the properties of the storage.
		//
		if (isset($infoArray['properties'])) {
			$properties = $infoArray['properties'];
			$this->createPropertiesXML($xml, $properties);
		}
		if (isset($infoArray['online'])) {
			$xml->element('Online', $infoArray['online'] == true ? "1" : "0");
		}
		if (isset($infoArray['mb_size'])) {
			$xml->element('MBSize', $infoArray['mb_size']);
		}
		if (isset($infoArray['mb_free'])) {
			$xml->element('MBFree', $infoArray['mb_free']);
		}
		if (isset($infoArray['mb_used'])) {
			$xml->element('MBUsed', $infoArray['mb_used']);
		}
		if (isset($infoArray['is_infinite'])) {
			$xml->element('IsInfinite', $infoArray['is_infinite']);
		}
		//
		// If any devices exist on the storage, display them.
		//
		if (isset($infoArray['devices'])) {
			$devices = $infoArray['devices'];
			if (count($devices) > 0) {
				foreach ($devices as $id) {
					if (is_array($deviceList) && isset($deviceList[$id])) {
						$name = $deviceList[$id];
						$networkID = sprintf("%d.%d", $dpuID, $id);
						$deviceInfo = $BP->get_device_info($id, $dpuID);
						if ($deviceInfo == false) {
							$deviceInfo = array('dev_name' => $name);
						}
						createDeviceXML($xml, $deviceInfo, $id, $networkID);
						//$xml->push('Device', array('ID' => $id, 'Name' => $name));
						//$xml->pop(); // Ddevice
					}
				}
			}
		}
		$xml->pop(); // Storage
	}
	
	//
	// Produces XML for the storage properties.
	//
	function createPropertiesXML($xml, $properties) {
		$xml->push("Properties");
		if (isset($properties['hostname'])) {
			$xml->element('Hostname', $properties['hostname']);
		}
		if (isset($properties['port'])) {
			$xml->element('Port', $properties['port']);
		}
		if (isset($properties['protocol'])) {
			$xml->element('Protocol', $properties['protocol']);
		}
		if (isset($properties['share_name'])) {
			$xml->element('Share', $properties['share_name']);
		}
		if (isset($properties['wwpn'])) {
			$xml->element('WWPN', $properties['wwpn']);
		}
		if (isset($properties['model'])) {
			$xml->element('Model', $properties['model']);
		}
		if (isset($properties['target'])) {
			$xml->element('Target', $properties['target']);
		}
		if (isset($properties['lun'])) {
			$xml->element('LUN', $properties['lun']);
		}
		if (isset($properties['vendor'])) {
			$xml->element('Vendor', $properties['vendor']);
		}
		if (isset($properties['username'])) {
			$xml->element('Username', $properties['username']);
		}
		if (isset($properties['internal_name'])) {
			$xml->element('InternalName', $properties['internal_name']);
		}
		if (isset($properties['max_size'])) {
			$xml->element('MaxSize', $properties['max_size']);
		}
		$xml->pop();
	}

	//
	// Given an integer storage type, returns the string corresponding to the type.
	//
	function getStorageType($nType) {
		$s = "unknown";
		switch ($nType) {
			case 0:
				$s = "Internal";
				break;
			case 1:
				$s = "iSCSI SAN";
				break;
			case 2:
				$s = "FC SAN";
				break;
			case 3:
				$s = "AOE SAN";
				break;
			case 4:
				$s = "NAS";
				break;
			case 5:
				$s = "Added Disk";
				break;
		}
		return $s;
	}	

	//
	// This function walks through the previously defined storage list and returns true if the specified disk is used already.
	//
	function isAllocated($BP, $thisName, $storage, $dpuID) {
		$used = false;
		if ($storage != false) {
			foreach ($storage as $id => $name) {
				if ($name == "Internal") {
					// Skip Internal disks
					continue;
				}
				$storageInfo = $BP->get_storage_info($id, $dpuID);
				if ($storageInfo !== false) {
					if (isset($storageInfo['type'])) {
						$type = (int)$storageInfo['type'];
						if ($this->getStorageType($type) == "Added Disk") {
							if (isset($storageInfo['properties'])) {
								$properties = $storageInfo['properties'];
								$internalName = isset($properties['internal_name']) ? $properties['internal_name'] : "";
								if ($thisName == $internalName) {
									$used = true;
									break;
								}
							}
						}
					}
				}
			}
		}
		return $used;
	}
	
	//
	// This function sees if the specified disks has an LVM partition; returns true if it does.
	//
	function inLVM($BP, $diskName, $lvmPartitions, $lvmInfo, $dpuID) {
		$inLVM = false;
		// First, check to see if the LVM information is relevant (not NULL) and
		// has free space.  If it does, we want to consider all these disks as expansion candidates.
		if ($lvmInfo != NULL && $lvmInfo !== false && $lvmInfo['gb_free'] > 0) {
			$inLVM = false;
		} else {
			if ($lvmPartitions === false) {
				$lvmPartitions = $BP->get_lvm_disks();
			}
			if ($lvmPartitions !== false) {
				foreach ($lvmPartitions as $partition) {
					// Use strstr for detection, as the partitioni will be of the form
					//  /dev/sda1
					// And the disk is of the form
					//  /dev/sda
					if (strstr($partition, $diskName) != false) {
						$inLVM = true;
						break;
					}
				}
			}
		}
		return $inLVM;
	}
}
	
?>
