<?php
// defines BP, xml, action, cookie
require '../header.php';

// general library functions
require $rootdir . 'bpl/library.php';
$lib = new Library();

// set DPU id
$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
$cid = isset($_GET['cid']) ? (int)$_GET['cid'] : -1;

define("OPERATIONS",     1);
define("BACKUPSTATS",    2);

switch ($action) {			
	// To list items present in the replication queue
	// On a source, it will only return the information related to the source
	// On a target, it will return all information related to the target and its sources. It will also
	// only return the first N number of queued items, aka start_pos will always be 0 on a target
	// Also aggregates information from get_backup_status and get_operation_list
	case "list":
			$vaultingConfigured = $BP->is_vaulting_configured($dpuID);
			if ($vaultingConfigured === true) {				
				$BP->buildResult($xml, false, "This system is vaulting, not replicating. " . 
					"To see the status of your vaulting jobs, go to Replication > Vaulting > Dashboard");
				echo($xml->getXml());
				break;
			}
			
			$startPosition = isset($_GET['start_pos']) ? (int)$_GET['start_pos'] : -1;
			$maxItems = isset($_GET['max']) ? (int)$_GET['max'] : -1;
			$includeFailed = isset($_GET['include_failed']) ? ($_GET['include_failed'] == '1') : false; 
			$includeActive = isset($_GET['include_active']) ? ($_GET['include_active'] == '1') : true;
			
			$filter = array('start_position' => $startPosition, 
							'max_items' => $maxItems,
							'include_failed' => $includeFailed,
							'include_active' => $includeActive);
					
			// contains a list of all current replications and operations related to replications
			$queue = array();
			$totalCount = 0;
			$systemCount = 0;
			
			$systems = $BP->get_system_list();
			if ($systems !== false) {
				$localID = $BP->get_local_system_id();
				$bVault = $BP->local_system_is_vault();
				$bMgr = $BP->local_system_is_mgr();
				$crossVaultDpuID = NULL;
				if ($bVault) {
					$crossVaultDpuID = getCrossVaultDpuID($BP, $systems[$localID], $systems);
				}
				
				if ($startPosition!=-1 && ($bVault && $localID == $dpuID)) {
					$startPosition = 0;
					// cannot page on a target, it will return the first N number of items as specified in "max_items"
					$filter['start_position'] = $startPosition;
				}				
				
				foreach ($systems as $systemID => $systemName) {
					if ($systemID == $dpuID || $dpuID == $localID) {
											
						// If we are looking at the target, skip the target.dpu system so we don't get duplicate data
						if ($bVault && ($dpuID == $localID) && ($systemID == $crossVaultDpuID)) {
							continue;
						}
						// If this system is a managed system, only look at the specified dpuID.
						if (!$bVault && $bMgr && ($systemID != $dpuID)) {
							continue;
						}
						$systemInfoArray =  array('systemID' => $systemID, 'systemName'=>$systemName);
						
						// -1 value means return all entries, do not adjust if maxItems is -1
						if ($maxItems!=-1) {
							// adjust max items so that we only return max items even if aggregating across all systems
							$filter['max_items'] = ($maxItems - $totalCount)>0 ? ($maxItems - $totalCount) : 0;
						}
						
						// count the number of systems potentially replicating to this target
						// target itself and target.dpu do not count
						if ($bVault && ($systemID != $localID)) {
							$systemCount = $systemCount + 1;
						}
								
						$result = $BP->get_replication_queue($filter, $systemID);
						if ($result !== false) {
							$count = isset($result['total_count']) ? $result['total_count'] : 0;
							$totalCount = $totalCount + $count;
							
							// get backup status only for queued entries
							$backupIDs = getBackupIDs($result["queued_entries"]);
							$backupStatsList = getBackupStatusList($backupIDs, $systemID);
							$tmpQueuedList = mergeReplicationInfo($result["queued_entries"], "backup_no",
												$backupStatsList, "id", BACKUPSTATS, $systemInfoArray);
							$queue = array_merge($queue, $tmpQueuedList);
							
							// get operation progress only for active entries
							$operationList = getOperationsList($systemID);
							// filter out all but replication operations
							$operationList = filterReplicationOperations($operationList);

							//Getting the backup status of the active replications
							$activeBackupIDs = getBackupIDs($result["active_entries"]);
							$activeBackupStatsList = getBackupStatusList($activeBackupIDs, $systemID);

							// We need the know the value of the encrypted, verified, disk_metadata, and synthesized fields in backupstatus
							$result["active_entries"] = decorateReplicationWithBackupStatusProperties( $result["active_entries"], $activeBackupStatsList);

							//$Log->writeVariable( $result["active_entries"] );
							
							$tmpActiveList = mergeReplicationInfo($result["active_entries"], "backup_no", 
												$operationList, "backup_id", OPERATIONS, $systemInfoArray);
							$queue = array_merge($queue, $tmpActiveList);
							// for local vault system, save any unmatched target operations
							if ($bVault && $localID == $dpuID) {
								$a = array();  // placeholder, only need to merge in systemInfoArray with these ops
								$tmpTargetOpList = mergeReplicationInfo($operationList, "backup_id", 
																$a, "nomatch", OPERATIONS, $systemInfoArray);
								$queue = array_merge($queue, $tmpTargetOpList);								
							}
						}
					}
				}
				
				// build XML with all the queue items
				if ($bVault && $localID == $dpuID) {
					// send system count for target, subtract the actual target from the count
					buildReplicationListXML($xml, $queue, $totalCount, $systemCount);
				}
				else {
					// no system count for a single system
					buildReplicationListXML($xml, $queue, $totalCount, -1);					
				}
			} else {
				$BP->buildResult($xml, $result);
			}
			echo($xml->getXml());
			break;

	// to add an item to the replication queue	
	case "add":
			$backup_no = isset($_GET['backup_no']) ? $_GET['backup_no'] : -1;
			$target = isset($_GET['target']) ? $_GET['target'] : -1;
			$result = $BP->add_to_replication_queue($backup_no, $target, $dpuID);
			if ($result !== false) {
				$BP->buildResult($xml, true);
			} else {
				$BP->buildResult($xml, $result);
			}
			echo($xml->getXml());
			break;
			
	case "is_in_queue":
			$backup_no = isset($_GET['backup_no']) ? $_GET['backup_no'] : -1;
			$target = isset($_GET['target']) ? $_GET['target'] : -1;
			$result = $BP->is_backup_in_replication_queue($backup_no, $target, $dpuID);
			if ($result !== false) {
				$xml->push('root');
				$xml->element( "Result", 1 );
				$xml->element( "QueueStatus", $result );
				$xml->pop();
			} else {
				$BP->buildResult($xml, $result);
			}
			echo($xml->getXml());
			break;
			
	// to update the order of queued replications
	case "update":
			$backup_nos = isset($_GET['backup_nos']) ? $lib->getStringArray($_GET['backup_nos']) : array();
			$targets = isset($_GET['targets']) ? $lib->getStringArray($_GET['targets']) : array();
			$queue_nos = isset($_GET['queue_nos']) ? $lib->getStringArray($_GET['queue_nos']) : array();
			
			$backupItems = array();
			foreach ($backup_nos as $key => $backup_no) {
				$entry = array();
				$entry['backup_no'] = (int)$backup_no;
				if (array_key_exists($key,$targets) && array_key_exists($key,$queue_nos)) {
					$entry['target_name'] = $targets[$key];
					$entry['queue_position'] = (int)$queue_nos[$key];
					$backupItems[] = $entry;
				}
			}
			
			$result = $BP->update_replication_queue($backupItems, $dpuID);
			if ($result !== false) {
				$BP->buildResult($xml, true);
			} else {
				$BP->buildResult($xml, $result);
			}
			echo($xml->getXml());
			break;
			
	// to change replication strategy
	case "change":
			$strategy = isset($_GET['strategy']) ? $_GET['strategy'] : -1;
			$result = $BP->set_replication_queue_strategy($strategy, $dpuID);
			if ($result !== false) {
				$BP->buildResult($xml, true);
			} else {
				$BP->buildResult($xml, $result);
			}
			echo($xml->getXml());
			break;

	// to delete replications from the replication queue
	case "delete":
			$backup_nos = isset($_GET['backup_nos']) ? $lib->getStringArray($_GET['backup_nos']) : array();
			$targets = isset($_GET['targets']) ? $lib->getStringArray($_GET['targets']) : array();
			
			$clients = isset($_GET['clients']) ? $lib->getStringArray($_GET['clients']) : array();
			$action = isset($_GET['action']) ? $_GET['action'] : "";
			
			if (count($backup_nos)>0 && count($targets)>0 && count($clients)>0) {
				// TODO sent in both backup targets and clients, return an error
			}
			
			$filter = array();
			// construct backup_targets
			$backup_targets = array();
			foreach ($backup_nos as $key => $backup_no) {
				$entry = array();
				$entry['backup_no'] = (int)$backup_no;
				if (array_key_exists($key,$targets)) {
					$entry['target_name'] = $targets[$key];
					$backup_targets[] = $entry;
				}
			}
			$filter["backup_targets"] = $backup_targets;
			$filter["clients"] = $clients;
			$filter["action"] = $action;
			
			$result = $BP->delete_from_replication_queue($filter, $dpuID);
			if ($result !== false) {
				$BP->buildResult($xml, true);
			} else {
				$BP->buildResult($xml, $result);
			}
			echo($xml->getXml());
			break;
}

	function getOperationsList($dpuID) {
		global $BP;
		$operationArray = array();
					
		// lmc - needed to get operations on non-local system.
		$systemID = $dpuID;
		
		$operationList = $BP->get_operation_list("", $systemID);
		
		/*
		if ($operationList === false) {
			continue;
		} else {
			foreach ($operationList as $operationInfo) {
				// Make sure we don't duplicate entries, particularly seen on a vault.  Don't have
				// to show "sides" of the operation since on a vault we roll up all statuses anyway.
				$bFound = false;
				foreach ($operationArray as $operationAlreadyFound) {
					if ($operationInfo['name'] == $operationAlreadyFound['name'] &&
						$operationInfo['source'] == $operationAlreadyFound['source'] &&
						$operationInfo['destination'] == $operationAlreadyFound['destination'] &&
						$operationInfo['backup_id'] == $operationAlreadyFound['backup_id'] &&
						$operationInfo['type'] == $operationAlreadyFound['type']) {
						$bFound = true;
						break;
					}
				}
				//
				// If not found already, add to our operations array for later display.
				//
				if (!$bFound) {
					$operationArray[] = $operationInfo;
				}
			}
		}*/
		
		return $operationList;
	}
	

	function getBackupIDs($replicationList) {
		$backupIDs = array();
		foreach ($replicationList as $replication) {
			$backupIDs[] = $replication['backup_no'];
		}
		return $backupIDs;
	}


	function getBackupStatusList ($backupIDs, $dpuID) {
		global $BP;
			
		$result_format = array();
		$result_format = array_merge($result_format, array('backup_ids' => $backupIDs, 'dpu_id'=>$dpuID, 'sync_msg'=>1));

		$backupStatusArray = $BP->get_backup_status($result_format);
		
		return $backupStatusArray;
	}

	//
	// Adds backup status properties to the replications that match them in order to allow for proper
	// display of these status data while in replication active and queued states
	function decorateReplicationWithBackupStatusProperties( &$replicationList, $backupStatusList ) {
		//global $Log;
		foreach ( $replicationList as $replicationKey=>$replication ) {
			foreach ( $backupStatusList as $backupKey=>$backupStatus ) {
				if ( $replication[ 'backup_no' ] == $backupStatus['id'] ) {
					$replication[ 'encrypted' ]     = $backupStatus['encrypted'];
					$replication[ 'verified' ]      = $backupStatus['verified'];
					$replication[ 'synthesized' ]   = $backupStatus['synthesized'];
					$replication[ 'disk_metadata' ] = $backupStatus['disk_metadata'];
					$replication[ 'replicated' ]    = $backupStatus['replicated'];
					$replicationList[ $replicationKey ] = $replication;
					//$Log->writeVariable( $replicationList );
					//$Log->writeVariable( "Breaking out of inner loop" );
					break;
				}
			}
		}
		return $replicationList;
	}
	
	/**
	This function takes in a list of current replications and a list of current operations and merges them together
	If no operation for a given replication is found, the replication info will be returned alone
	$replicationList = list of current replications
	$replicationMergeKey = key from the replication list to match up with the other list
	$mergeList = list to match up with the replication list (could be operation list, sds stats list)
	$mergeKey = key from the merge list to match up with the replication list
	$mergeType = type of the merge, currently we only merge results from the operation_progress table and the sds_stats table
	*/
	function mergeReplicationInfo ($replicationList, $replicationMergeKey, &$mergeList, $mergeKey, $mergeType, $systemInfoArray) {
		$mergedReplicationInfo[] = array();
		$returnList = $replicationList;
		foreach ($returnList as $returnListKey=>$replication) {
			$returnList[$returnListKey] = array_merge($returnList[$returnListKey], $systemInfoArray);			
			
			foreach ($mergeList as $mergeListKey=>$merge) {
				if ($replication[$replicationMergeKey] === $merge[$mergeKey]) {
					
					// for OPERATIONS, return unmatched operations
					// for BACKUPSTATS, return sync msg for failed replications
					
					if (isset($replication['status'])) {
						$merge['status'] = $replication['status'];
					}
					
					// for BACKUPSTATS merge, return sync msg for failed replications
					if ($mergeType == BACKUPSTATS) {
						if (isset($merge['sync_message'])) {
							// if replication has failed before, include sync_msg in the returned data
							if (isset($replication['status']) && $replication['status']==4) {
								$sync_msg = $merge['sync_message'];
								$returnList[$returnListKey] = array_merge($returnList[$returnListKey], array('sync_message'=>$sync_msg));
							}
						}
						else {
							// a match between replication and the merge array was found, merge them into a new structure
							$returnList[$returnListKey] = array_merge($returnList[$returnListKey], $replication, $merge);
						}
					}
					
					
					// for OPERATIONS merge, remove matched operations
					if ($mergeType == OPERATIONS) {
						// a match between replication and the merge array was found, merge them into a new structure
						$returnList[$returnListKey] = array_merge($returnList[$returnListKey], $merge);
						// unset the matched up value from mergeList so that only the unmatched ones remain and can be returned
						unset($mergeList[$mergeListKey]);
						// break, there is only one operation per backup number. no reason to search any further

						break;
					}
				}
			}
		}
		
		return $returnList;
	}

	// Gets string "true" or "false" depending on passed in value.
	function getStringTrueOrFalse( $value ) {
		if ( $value == 1 || 
		     $value === true || 
			 strtolower( $value ) === "true" || 
			 $value === "1" ) {
			 
			return "true";
		} else {
			return "false";
		}
	}

	function buildReplicationListXML($xml, $replication_list, $total_count, $systemCount, $showRoot = true) {
		if ($showRoot == true) $xml->push('root');		
		
		if ($systemCount!=-1) {
			$xml->emptyelement("Queue", array("TotalCount" => $total_count, "SystemCount" => $systemCount));
		}
		else {
			$xml->emptyelement("Queue", array("TotalCount" => $total_count));			
		}
		
		foreach ($replication_list as $replication) {
			$day = ""; 
			$time = ""; 
			$source = "";
			$target = "";
			$status = -1;
			
			if (count($replication)<=0) {
				// skip any replications that have no attributes
				continue;
			}
			$xml->push('Replication');
			
			if (isset($replication['systemName'])) {
				$xml->element("SystemName", $replication['systemName']);
			}
			if (isset($replication['systemID'])) {
				$xml->element("SystemID", $replication['systemID']);
			}
			
			if (isset($replication['queue_position'])) {
				$xml->element("QueuePosition", $replication['queue_position']);
			}
			
			if (isset($replication['target_name'])) {
				$target = $replication['target_name'];
				$xml->element("TargetName", $target);
			}						
			else if (isset($replication['destination'])) {
				$target = $replication['destination'];
				$xml->element("TargetName", $target);
			}
			
			if (isset($replication['source'])) {
				$source = $replication['source'];
				$xml->element("Source", $source);
			}
			// check whether this is a source operation or a target operation
			if (isset($source) && strlen($source)>0 &&strcasecmp($source,$target) == 0) {
				// source and target are the same, it's a target operation
				$xml->element("OperationSource", "target");
				// add status 16 (in progress) for target operations
				$status = 16;
				$xml->element("Status", 16);
			}
			else {
				// source operation
				$xml->element("OperationSource", "source");
			}
						
			if (isset($replication['backup_no'])) {
				// get_replication_queue_list returns the "backup_no" element
				$xml->element("BackupNumber", $replication['backup_no']);
			} else if (isset($replication['backup_id'])) {
				// get_operation_list returns the "backup_id" element
				$xml->element("BackupNumber", $replication['backup_id']);
			}
			
			if (isset($replication['status'])) {
				$status = $replication['status'];
				$xml->element("Status", $replication['status']);
			}
			
			// return the overall start time for replication that is in progress
			// if rep_start has a value of 0, do not report it
			if ( $status == 16 && isset($replication['rep_start']) && $replication['rep_start']!=0 ) {
				// convert timestamp to local time/day
				$day = date('m/d/Y', $replication['rep_start']);
				$time = date(TIME_FORMAT_12HR, $replication['rep_start']);
				$xml->element('OverallStartDate', $day);
				$xml->element('OverallStartTime', $time);
			}
			
			if (isset($replication['type'])) {
				$syncBackupType = $replication['type'];
				switch ($syncBackupType) {
				// Convert sync types to known names.
					case 'MSExch Full':
					case 'MSExch Diff':
					case 'MSExch Incr':
					case 'MSSQL Full':
					case 'MSSQL Diff':
					case 'MSSQL Trans':
					case 'ESX VM Full':
					case 'ESX VM Diff':
					case 'ESX VM Incr':
						$syncType = getBackupTypeString('securesync ' . strtolower($syncBackupType));
						$xml->element('Type', $syncType, true);
					break;
				
					default:
						$xml->element('Type', getBackupTypeString($replication['type'], true));
					break;
				}
			}
			
			if (isset($replication['client_name'])) {
				$xml->element("ClientName", $replication['client_name']);
			}
			else if (isset($replication['on_behalf_of'])) {
				$xml->element("ClientName", $replication['on_behalf_of']);
			}			
			
			if (isset($replication['instance_name'])) {
				$xml->element("InstanceName", $replication['instance_name']);
			}
			if (isset($replication['current_size'])) { // size in MB
				$xml->element("CurrentSize", $replication['current_size']);
			}
			if (isset($replication['final_size'])) { // size in MB
				// get_operation_list returns the "final_size" element
				$xml->element("FinalSize", $replication['final_size']);
			} else if (isset($replication['size'])) {
				// get_backup_status returns the "size" element
				$xml->element("FinalSize", $replication['size']);
			}
			if (isset($replication['start_time'])) {
				// convert timestamp to local time/day
				$day = date('m/d/Y', $replication['start_time']);
				$time = date(TIME_FORMAT_12HR, $replication['start_time']);
				$xml->element('StartDate', $day);
				$xml->element('StartTime', $time);
			}
			if (isset($replication['current_time'])) {
				// convert timestamp to local time/day
				$day = date('m/d/Y', $replication['current_time']);
				$time = date(TIME_FORMAT_12HR, $replication['current_time']);
				$xml->element('CurrentDate', $day);
				$xml->element('CurrentTime', $time);
			}
			
			if (isset($replication['elapsed_time'])) {
				$xml->element('ElapsedTime', $replication['elapsed_time']);
			}
			else if (isset($replication['current_time'])) {
				$startTime = -1;
				if (isset($replication['rep_start']) && $replication['rep_start'] != 0) {
					$startTime = $replication['rep_start'];
				}
				else if (isset($replication['start_time'])) {
					$startTime = $replication['start_time'];
				}
				
				if ($startTime != -1) {				
					$elapsedTime = $replication['current_time'] - $startTime;
					$xml->element('ElapsedTime', $elapsedTime);
				}
			}
			
			$operationName = (isset($replication['name'])) ? $replication['name'] : "Target Processing";
			$xml->element('OperationName', getFormattedName($operationName));
			
			
			// various other attributes			
			if (isset($replication['encrypted'])) {
				$xml->element("Encrypted", getStringTrueOrFalse( $replication['encrypted'] ));
			}
			if (isset($replication['verified'])) {
				$xml->element("Verified", getStringTrueOrFalse( $replication['verified'] ));
			}
			if (isset($replication['verify_status'])) {
				$xml->element("Verify_status", $replication['verify_status']);
			}
			if (isset($replication['synthesized'])) {
				$xml->element("Synthesized", getStringTrueOrFalse( $replication['synthesized'] ));
			}
			if (isset($replication['synth_capable'])) {
				$xml->element("Synth_capable", $replication['synth_capable']);
			}
			if (isset($replication['disk_metadata'])) {
				$xml->element("Disk_metadata", getStringTrueOrFalse( $replication['disk_metadata'] ));
			}
			if (isset($replication['replicated'])) {
				$xml->element("Replicated", getStringTrueOrFalse( $replication['replicated'] ));
			}
			if (isset($replication['sync_status'])) {
				$xml->element("Sync_status", $replication['sync_status']);
			}
			if (isset($replication['sync_message'])) {
				$xml->element("SyncMessage", $replication['sync_message']);
			}
			
			$xml -> pop(); // Replication
		}
		
		if ($showRoot == true) $xml->pop();
	}
	
	
	//
	// This function returns a formatted string of the backup type field.
	// If returnUnformattedName is true, then this function will return the unformatted
	// name if a match is not found, if false then it will return null
	//
	function getFormattedName($field, $returnUnformattedName=true)
	{
		$name = NULL;
		if ($returnUnformattedName == true) {
			$name = $field;
		}
		
		switch ($field) {					
			// new keywords for replication
			case "prepare":
					// on source side
					$name = "Prepare (1/4)";
					break;
			case "replicate":
					// on source side
					$name = "Replicate (2/4)";
					break;
			case "wait":
					// on source side
					$name = "Target Processing (4/4)";
					break;
			case "rebuild":
					// on target side
					$name = "Rebuild (4/4)";
					break;
			case "no_activity":
					$name = "No Activity";
					break;
			case "processing":
					// on source side, after the replicate state but before the wait state
					$name = "Processing (3/4)";
					break;
		}
		
		return $name;
	}
	
	function filterReplicationOperations($operationList) {
		$newOperationList = array();
		foreach ($operationList as $key=>$operation) {
			$formattedName = getFormattedName($operation['name'], false);
			// skip any operations that don't have a match
			if ($formattedName!=NULL) {
				$newOperationList[] = $operation;
			}
		}
		
		return $newOperationList;
	}

$BP->destroyCookie();
?>