<?php
//
// This function creates an xml customer list.
//
require_once('credentials.php');

// See if we only want to match a particular source system.
$sourceID = isset($_GET['src']) ? (int)$_GET['src'] : -1;
$localID = $BP->get_local_system_id();

function buildCustomerList($BP, $xml, $bShowRoot = true, $systemsArray = NULL)
{
	$bValid = true;
	$customers = $BP->get_customer_list();
	if ($customers === false) {
		$bValid = false;
	} else {
		if ($bShowRoot) {
			$xml->push("root");
		}
		foreach ($customers as $id => $name) {
			$xml->push("Customer", array("Name" => $name, "ID" => $id));
			$locationList = $BP->get_location_list($id);
			if ($locationList === false) {
				$bValid = false;
				break;
			} else {
				foreach ($locationList as $locationID => $locationName) {
					$locationInfo = $BP->get_location_info($locationID);
					if ($locationInfo === false) {
						$bValid = false;
						break;
					} else {
						$xml->push("Location", array("Name" => $locationName, "ID" => $locationID));
						$xml->element("Where", $locationName);
						if (array_key_exists('latitude', $locationInfo) && $locationInfo['latitude'] != 0) {
							$xml->element("Latitude", $locationInfo['latitude']);
						} else {
							$xml->element("Latitude", "");
						}
						if (array_key_exists('longitude', $locationInfo) && $locationInfo['longitude'] != 0) {
							$xml->element("Longitude", $locationInfo['longitude']);
						} else {
							$xml->element("Longitude", "");
						}
						listSystemsForCustomerLocation($xml, $BP, $id, $locationID, $systemsArray);
						$xml->pop(); // location
					}
				}
			}
			$xml->pop(); // customer
		}
		if ($bShowRoot) {
			$xml->pop(); // root
		}
	}
	if (!$bValid) {
		$xml->clear(false);
		$BP->buildResult($xml, false);
	}
	
	return $bValid;
}

//
// This function builds an array of system ID, name and info.
//
function buildSystemsArray($BP) {
	
	global $sourceID;
	
	$systemsArray = array();
	$systems = $BP->get_system_list();
	if ($systems !== false) {
		foreach ($systems as $id => $name) {
			//
			// If we want a match, skip if not a match.
			//
			if ($sourceID != -1 && $id != $sourceID) {
				continue;
			}
			$systemInfo = $BP->get_system_info($id);
			if ($systemInfo !== false) {
				$systemsArray[] = $systemInfo;
			}
		}
	} else {
		$systemsArray = NULL;
	}
	return $systemsArray;
}

//
// This function creates an xml system list.
//
function buildSystemList($BP, $xml, $bShowRoot = true, $systemsArray = NULL, $showDetails = true)
{
	global $localID;
	$bVault = $BP->LocalSystemIsVault();
	$bMgr = $BP->local_system_is_mgr();
	
	$bValid = true;
	if ($systemsArray == NULL || (count($systemsArray) == 0)) {
		$systemsArray = buildSystemsArray($BP);
	}
	if ($systemsArray != NULL && count($systemsArray) > 0) {
		if ($bShowRoot) {
			$xml->push("root");
		}

		foreach ($systemsArray as $systemInfo) {
			$name = $systemInfo['name'];
			$id = $systemInfo['id'];
			$xml->push("System", array("Name" => $name, "ID" => $id));

			$SystemType = getSystemType($BP, $id, $localID, $bVault, $bMgr);
			$xml->element("Type", $SystemType);
			$xml->element("Local", ($id == $localID) ? "1" : "0");
			$host = array_key_exists('host', $systemInfo) ? $systemInfo['host'] : "";
			
			$customerName = array_key_exists('customer_name', $systemInfo) ? $systemInfo['customer_name'] : "Default Customer";
			$customerID = array_key_exists('customer_id', $systemInfo) ? $systemInfo['customer_id'] : "";
			
			$xml->element("Host", $host);
			if (array_key_exists('long_name', $systemInfo)) {
				$xml->element("LongName", $systemInfo['long_name']);
			}
			$xml->element("Customer", $customerName, array(	"Name" => $customerName,
															"ID" => $customerID));
																		
			$myLocation = array_key_exists('location_name', $systemInfo) ? $systemInfo['location_name'] : "";				   
			$myLocationID = array_key_exists('location_id', $systemInfo) ? $systemInfo['location_id'] : "";
												 
			$xml->element("Location", $myLocation, array("Name" => $myLocation,
														 "ID" => $myLocationID));

			if (array_key_exists('role', $systemInfo)) {
				$xml->element("Role", $systemInfo['role']);
			}															 
			if (array_key_exists('version', $systemInfo)) {
				$xml->element("Version", $systemInfo['version']);
			}
			if (array_key_exists('storage_id', $systemInfo)) {
				$xml->element('Storage', $systemInfo['storage_id']);
			}
			if (array_key_exists('device_id', $systemInfo)) {
				$xml->element('DeviceID', $systemInfo['device_id']);
			}
			
			$isCloudhookSupported = $BP->cloudhook_supported($id);
			
			$xml->element("CloudhookSupported",$isCloudhookSupported);

			if ($showDetails) {
				//
				// See if dedup and incremental backups are supported (block level dedup).
				//
				$dedup = $BP->dedup_supported($id);
				$incr = ($dedup == 'block') ? 1 : 0;
				$xml->element("Incr", $incr);
				//
				// See if the system uses SSL by looking in the master.ini.
				//
				$sslResult = $BP->get_ini_value("Configuration Options", "UseSSL", $id);
				if ($sslResult == false) {
					// If it fails, we'll presume no SSL, but do log the failure.
					global $Log;
					$Log->writeVariable($BP->getError());
				}
				$xml->element('UseSSL', ($sslResult != false && (strtolower($sslResult) == 'yes')) ? 1 : 0);
				//
				// See if a virtual or physical appliance.
				//
				$virtualResult = $BP->is_virtual($id);
				if ($virtualResult === -1) {
					// If the API fails, we'll have to presume something, so not virtual.
					global $Log;
					$Log->writeVariable($BP->getError());
				}
				$xml->element('Virtual', $virtualResult == true ? 1 : 0);

                //
                //Check if the system is OnMetal
                //
                $OnMetal = $BP ->get_capabilities("Unitrends-Metal",$id);
                $xml -> element('OnMetal', (is_array($OnMetal) and array_key_exists('Unitrends-Metal', $OnMetal) and $OnMetal['Unitrends-Metal'] != false) ? 1 : 0);

				listClients($xml, $BP, $id);
			}

			$xml->pop(); // system
		}
		if ($bShowRoot) {
			$xml->pop(); // root
		}
	} else {
		$bValid = false;
	}
	
	if (!$bValid) {
		$xml->clear(false);
		$BP->buildResult($xml, false);
	}
	return $bValid;
}

//
// This function creates an xml of a system for a given customer and location.
//
function listSystemsForCustomerLocation($xml, $BP, $customerID, $locationID, $systemsArray)
{
	if ($systemsArray == NULL || (count($systemsArray) == 0)) {
		$systemsArray = buildSystemsArray($BP);
	}
	if ($systemsArray != NULL && count($systemsArray) > 0) {
		foreach ($systemsArray as $systemInfo) {
			if (array_key_exists('customer_id', $systemInfo) && array_key_exists('location_id', $systemInfo)) {
				if ($systemInfo['customer_id'] == $customerID && $systemInfo['location_id'] == $locationID) {
					$xml->push("System", array("Name" => $systemInfo['name'], "ID" => $systemInfo['id']));
					$xml->pop();
				}
			}
		}
	}
}

//
// This function returns the string type of the system.
//
function getSystemType($BP, $idDPx, $localID, $bVault, $bMgr)
{
	$systemType = "DPU";

	if ($bVault) {
		if ($localID == $idDPx) {
			$systemType = "DPV";
		}
	} else if ($bMgr) {
		if ($localID == $idDPx) {
			$systemType = "MDPU";
		}
	}	
	return $systemType;
}

//
// This functin creates the XML of the clients on a particular system.
//
function listClients($xml, $BP, $systemID)
{
	global $grandClientView;
	global $localID;
	
	$returnValue = true;
	if ($grandClientView && $systemID != $localID) {
		$clientList = $BP->get_grandclient_list($systemID);
		$systemID = $localID;
	} else {
		$clientList = $BP->get_client_list($systemID);
	}
	if ($clientList !== false) {
		foreach ($clientList as $id => $name) {
			// Generate a "unique" name for the client attribute, combining the DPU ID and client ID.
			$networkID = sprintf("%d.%d", $systemID, $id);
			$clientInfo = $BP->get_client_info($id, $systemID);
			if ($clientInfo !== false) {
				$returnValue = listClient($xml, $clientInfo, $id, $networkID);
			} else {
				global $Log;
				$message = " listClients: not get information for client: id = " . $id . ", name = " . $name . ".\n";
				$message = $BP->getError() . ":" . $message;
				$Log->writeError($message, true);
			}
		}
	}
}

//
// This functin creates the XML of a client.
//
function listClient($xml, $clientInfo, $id, $networkID)
{
	$tmp = explode(".", $networkID);
	$dpuID = $tmp[0];
	
	$Credentials = NULL;
	if (isset($clientInfo['credentials']) && count($clientInfo['credentials']) > 0) {
		$Credentials = new Credentials();
	}	
	createClientXML($xml, $clientInfo, $id, $networkID, true, $Credentials, $dpuID);
}

//
// This functin creates the XML of the devices on a particular system.
//
function listDevices($xml, $BP, $systemID)
{
	$returnValue = true;
	$deviceList = $BP->get_device_list($systemID);
	if ($deviceList !== false) {
		foreach ($deviceList as $id => $name) {
			// Generate a "unique" name for the device attribute, combining the DPU ID and device ID.
			$networkID = sprintf("%d.%d", $systemID, $id);
			$deviceInfo = $BP->get_device_info($id, $systemID);
			if ($deviceInfo !== false) {
				createDeviceXML($xml, $deviceInfo, $id, $networkID);
			} else {
				global $Log;
				$message = " listDevices: could not get information for device: id = " . $id . ", name = " . $name . ".\n";
				$message = $BP->getError() . ":" . $message;
				$Log->writeError($message, true);
			}
		}
	}
}

function createLicenseXML($BP, $xml, $licenseInfo, $dpuID)
{
	// Output fields that are applicable to both demo and full licenses.
	$xml->push("License");
	
	$assetTag = $BP->get_asset_tag($dpuID);
	if($assetTag !== false){
		$xml->element("AssetTag", $assetTag);
	}
	if (array_key_exists('product', $licenseInfo)) {
		$xml->element("Product", $licenseInfo['product']);
	}
	if (array_key_exists('version', $licenseInfo)) {
		$xml->element("Version", $licenseInfo['version']);
	}
	if (array_key_exists('serial_number', $licenseInfo)) {
		$xml->element("SerialNumber", $licenseInfo['serial_number']);
	}
	if (array_key_exists('vendor_information', $licenseInfo)) {
		$xml->element("Vendor", $licenseInfo['vendor_information']);
	}
	if (array_key_exists('comment', $licenseInfo)) {
		$xml->element("Comment", $licenseInfo['comment']);
	}
	if (array_key_exists('license_type', $licenseInfo)) {
		$xml->element("Type", $licenseInfo['license_type']);
	}
	if (array_key_exists('user_string', $licenseInfo)) {
		$xml->element("UserString", $licenseInfo['user_string']);
	}
	if (array_key_exists('daemon_host_id', $licenseInfo)) {
		$xml->element("DaemonHostID", $licenseInfo['daemon_host_id']);
	}
	if (array_key_exists('install_date', $licenseInfo)) {
		$xml->element("InstallDate", $licenseInfo['install_date']);
	}
			
	// The following are demo license fields only.
	if (array_key_exists('max_inst_date', $licenseInfo)) {
		$xml->element("MaxInstallDate", $licenseInfo['max_inst_date']);
	}
	if (array_key_exists('demo_period', $licenseInfo)) {
		$xml->element("DemoPeriod", $licenseInfo['demo_period']);
	}
	if (array_key_exists('demo_installs', $licenseInfo)) {
		$xml->element("DemoInstalls", $licenseInfo['demo_installs']);
	}
	$nVC = $nRC = 0;
    $allowDPU = true;  // If $allowDPU is true, the system role can be set to Backup System
    $allowDPV = true;  // If $allowDPV is true, the system role can be set to Vault
    // If $allowDPU and $allowDPV are both true, the system role can be set to Cross-Vault
	if (array_key_exists('feature_string', $licenseInfo)) {
		$xml->element("FeatureString", $licenseInfo['feature_string']);
		$friendlyFeatureString = formatFeatureString($licenseInfo['feature_string'], $nRC, $nVC, $allowDPU, $allowDPV);
		$xml->element('FriendlyFeature', $friendlyFeatureString);
	}
	$xml->element('VC', $nVC);
	$xml->element('RC', $nRC);
    $xml->element('AllowDPU', ($allowDPU === true ? "1" : "0") );
    $xml->element('AllowDPV', ($allowDPV === true ? "1" : "0") );
	
	if (array_key_exists('expiration_date', $licenseInfo)) {
		$xml->element("ExpirationDate", $licenseInfo['expiration_date']);
	}
	if (array_key_exists('daemon_host', $licenseInfo)) {
		$xml->element("DaemonHost", $licenseInfo['daemon_host']);
	}
	if (array_key_exists('daemon_ip_port', $licenseInfo)) {
		$xml->element("DaemonIP", $licenseInfo['daemon_ip_port']);
	}
	if (array_key_exists('users', $licenseInfo)) {
		$xml->element("Users", $licenseInfo['users']);
	}
	if (array_key_exists('client_lock', $licenseInfo)) {
		$xml->element("ClientLock", $licenseInfo['client_lock']);
	}
	if (array_key_exists('key', $licenseInfo)) {
		$xml->element("Key", $licenseInfo['key']);
	}
	if (array_key_exists('class', $licenseInfo)) {
		$xml->element("Class", $licenseInfo['class']);
		$description = mapLicenseDescription($licenseInfo['class']);
		$xml->element("Description", $description);
	} 

	$xml->pop(); //License
}

function mapLicenseDescription($class){
	switch ($class) {
		case "UNREG":
			$description = "Unregistered";
			break;
        case "CE":
		case "FREE":
			$description = "Free Edition";
			break;
		case "NFR":
			$description = "Not For Resale";
			break;
		case "TRIAL":
			$description = "Trial Edition";
			break;
		case "ENTRB":
			//$description = "Enterprise Edition (per TB)";
			$description = "Enterprise Edition"; 
			break;
		case "ENTRR":
			//$description = "Enterprise Edition (per Resource)";
			$description = "Enterprise Edition"; 
			break;
		case "PHY":
			$description = "Enterprise Edition with Physical Unit";
			break;
		default:
			$description = "No Description";
			break;
	}
	return $description;
}

//feature string possibilities:
//Physical appliance (no license class) J=120,CSS,MUX=10,VC=0,RC=300G,D2D=300G,ENC,ADX
//Free License							FREE,MUX=10,VM=4,VC=0,RC=inf,D2D=inf,ENC,ADX
//TRAIL LICENSE(WITH NEVER EXPIRE) 		TRAIL,MUX=10,VC=inf,RC=inf,D2D=inf,ENC,ADX
//NOT FOR RESELLER (NFR) LICENSE:		NFR,MUX=10,VM=4,VC=inf,RC=inf,D2D=inf,ENC,ADX
//ENTERPRISE RESOURCE LICENSE			ENTRR,MUX=10,VC=inf,D2D=inf,ENC,ADX,WRK=500,SOC=2,SRV=1,APP=1
//ENTERPRISE BYTE LICENSE				ENTRB,MUX=10,VC=inf,RC=300G,D2D=inf, ENC, ADX

function formatFeatureString($FeatureString, &$nRC, &$nVC, &$allowDPU = true, &$allowDPV = true) {
	//global $Log;
	$stringArray = explode(",", $FeatureString);
	for ($i = 0; $i < count($stringArray); $i++) {
		$LicenseElement = $stringArray[$i];
		$tokenArray = explode('=', $LicenseElement);
		if (count($tokenArray) > 0) {
			if ($tokenArray[0] == "J") {
				//$Log->writeVariable("token" . $tokenArray[0]);
				$friendlyFeatureString = formatPhysicalLicense($stringArray, $nRC, $nVC, $allowDPU, $allowDPV);
			} else {
				//$Log->writeVariable("token" . $tokenArray[0]);
				$friendlyFeatureString = formatVirtualLicense($stringArray, $allowDPU, $allowDPV);
			}
			
		}
		return $friendlyFeatureString;
	}
}

//Physical appliance (no license class) J=120,CSS,MUX=10,VC=0,RC=300G,D2D=300G,ENC,ADX
function formatPhysicalLicense($stringArray, &$nRC, &$nVC, &$allowDPU = true, &$allowDPV = true) {
	global $Log;
//	$Log->writeVariable("physical");
	$Size = "";
	$D2Dsize = "";
	$RCsize = "";
	$VCsize = "";
	$Type = "";
	$ENC = "";
	$EXT = "";
	$ADX = "";
	
	$features = "";

    $D2DtokenPresent = false;
    $VCtokenPresent = false;
	
	for ($i = 0; $i < count($stringArray); $i++) {
		$LicenseElement = $stringArray[$i];
		$tokenArray = explode('=', $LicenseElement);
		if (count($tokenArray) > 0) {
			//$Log->writeVariable($tokenArray);
			switch($tokenArray[0]) {
				case "D2D":
                    $D2DtokenPresent = true;
					$temp = explode ("G", $tokenArray[1]);
					$D2Dsize = (int)$temp[0];
                    $allowDPU = ($D2Dsize !== 0);
					break;
				case "RC":	
					$temp = explode ("G", $tokenArray[1]);
					$RCsize = (int)$temp[0]; 
					$nRC = $RCsize;
					$features = $features . " RC " . $RCsize;
					break; 
				case "VC":
                    $VCtokenPresent = true;
					$temp = explode ("G", $tokenArray[1]);
					$VCsize = (int)$temp[0]; 
					$nVC = $VCsize;
					$features = $features . " VC " . $VCsize;
                    $allowDPV = ($VCsize !== 0);
					break;
				case "ENC":
					$ENC =  ", Encryption";
					break;
				case "EXT":
					$EXT = ", External Storage";
					break;
				case "ADX":
					$ADX = ", Archiving";			
					break;
				
			}
				
		}
		
		if(($RCsize > 0) && ($VCsize <= 0)) {
			$Type = "System";
			$Size = $RCsize;
		}
		
		if(($RCsize <= 0) && ($VCsize > 0)) {
			$Type = "Vault";
			$Size = $VCsize;
		} 
		
		if(($RCsize > 0) && ($VCsize > 0)) {
			$Type = "Vault, Backup";
			$Size = $D2Dsize;
		}
		
		$features = $Size . "G " . $Type . $ENC . $EXT . $ADX ; 
	
	}

    if ( $D2DtokenPresent === false ) {
        $allowDPU = false;
    }

    if ( $VCtokenPresent === false ) {
        $allowDPV = false;
    }

	return $features;
}

//Free License							FREE,MUX=10,VM=4,VC=0,RC=inf,D2D=inf,ENC,ADX
//TRAIL LICENSE(WITH NEVER EXPIRE) 		TRAIL,MUX=10,VC=inf,RC=inf,D2D=inf,ENC,ADX
//NOT FOR RESELLER (NFR) LICENSE:		NFR,MUX=10,VM=4,VC=inf,RC=inf,D2D=inf,ENC,ADX
//ENTERPRISE RESOURCE LICENSE			ENTRR,MUX=10,VC=inf,D2D=inf,ENC,ADX,WRK=500,SOC=2,SRV=1,APP=1
//ENTERPRISE BYTE LICENSE				ENTRB,MUX=10,VC=inf,RC=300G,D2D=inf, ENC, ADX

function formatVirtualLicense($stringArray, &$allowDPU = true, &$allowDPV = true) {
	global $Log;
	//$Log->writeVariable("vRecovery");
	$Size = "";
	$Type = "";
	$ENC = "";
	$EXT = "";
	$ADX = "";
	$D2Dsize = "";
	$RCsize = "";
	$VCsize = "";
	
	$features = "";

    $D2DtokenPresent = false;
    $VCtokenPresent = false;
	
	for ($i = 0; $i < count($stringArray); $i++) {
		$LicenseElement = $stringArray[$i];
	//	$Log->writeVariable($LicenseElement);
		$LicenseType = $LicenseElement[0];
		$tokenArray = explode('=', $LicenseElement);

		if (count($tokenArray) > 0) {
			//$Log->writeVariable($tokenArray);

			switch($tokenArray[0]) {
				case "D2D":
                    $D2DtokenPresent = true;
					$temp = explode ("G", $tokenArray[1]);
					$D2Dsize =  getSize($temp[0]);
                    $allowDPU = ($D2Dsize !== 0 && $D2Dsize !== '0G');
					break;
				case "RC":
					$temp = explode ("G", $tokenArray[1]);
					$RCsize = getSize($temp[0]); 
					break; 
				case "VC":
                    $VCtokenPresent = true;
					$temp = explode ("G", $tokenArray[1]);
					$VCsize = getSize($temp[0]);
                    $allowDPV = ($VCsize !== 0 && $VCsize !== '0G');
					break;
				case "VM":
					$VMs = $tokenArray[1];	
				case "ENC":
					$ENC =  ", Encryption";
					break;
				case "EXT":
					$EXT = ", External Storage";
					break;
				case "ADX":
					$ADX = ", Archiving";			
					break;
			}
		}		
	}
	
	if ($VCsize == "INF" && ($RCsize == "INF" || $D2Dsize == "INF")) {
		// All are infinite; backups size is either RC or D2D.
		$Type = 'Unlimited Replication or Backups';
	} else if ($VCsize == "INF" && $RCsize !== "INF" && $D2Dsize !== "INF") {
		$Type = 'Unlimited Replication';
	} else if (($RCsize == "INF" || $D2Dsize == "INF") && $VCsize !== "INF") {
		$Type = 'Unlimited Backups';
	} else {
		// None are infinite, get size.
		$fixedSizeType = calcFixedSize($D2Dsize, $RCsize, $VCsize);
		$features = $fixedSizeType;
	}
	
	$features .= $Type . $ENC . $ADX;

    // If the D2D token is not present, then the system cannot be a DPU
    if ( $D2DtokenPresent === false ) {
        $allowDPU = false;
    }

    // If the VC token is not present, then the system cannot be a DPV
    if ( $VCtokenPresent === false ) {
        $allowDPV = false;
    }
	
	return $features;
}

function getSize($element) {
	if (strtoupper($element) == 'INF') {
		return strtoupper($element);
	} else {
		return (float)$element . "G";
	}
}

function calcFixedSize($D2Dsize, $RCsize, $VCsize) {
	if(($RCsize > 0) && ($VCsize <= 0)) {
		$Type = " Backup";
		$Size = $RCsize;
	}
		
	if(($RCsize <= 0) && ($VCsize > 0)) {
		$Type = "Vault";
		$Size = $VCsize;
	} 
		
	if(($RCsize > 0) && ($VCsize > 0)) {
		$Type = "Vault, Backup";
		$Size = $D2Dsize;
	}
	return $Size . $Type;
}

?>
