<?php
// defines BP, xml, action, cookie
require 'header.php';

// defines functions for URL parsing.
require('urloptions.php');

// defines functions for backup status processing.
require('basebackupstatus.php');

$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();

$localID = $BP->get_local_system_id();
$bVault = $BP->local_system_is_vault();
$bMgr = $BP->local_system_is_mgr();

// add the dpu state if vaulting.
$DPUSTATE = "securesync dpustate";
// add the appliance agent if replicating.
$APP_METADATA = "securesync system metadata";

define('SYNC_XML_TAG', 'SecureSyncStatus');
$tag = SYNC_XML_TAG;

switch ($action) {

	case "sync":
	case "replicate":
		//
		// Default to retrieve all backup types; Override the default backup types for SQL and securesync reports.
		//
		$backupTypes = $GLOBAL_BACKUP_TYPES;
		if ($action == 'sync') {
			$backupTypes = $GLOBAL_SS_TYPES;
			$backupTypes[] = $DPUSTATE;
		} else if ($action == 'replicate') {
			$backupTypes = $GLOBAL_SS_TYPES;
			$backupTypes[] = $APP_METADATA;
		}

		$result_format = array('type' => $backupTypes, 'sync_msg' => 1);	
		$attributes = getTimeArguments(true);
		$result_format = array_merge($result_format, $attributes);
		$originalFormat = $result_format;
		
		$systems = $BP->get_system_list();
		
		$crossVaultDpuID = -1;
		if ($bVault) {
			$crossVaultDpuID = getCrossVaultDpuID($BP, $systems[$localID], $systems);
		}
		
		if ($systems !== false) {
		
			$xml->push('BackupInformation');
			foreach ($systems as $systemID => $systemName) {
				if ($systemID == $dpuID || $dpuID == $localID) {					
					// If this system is a vault and we are looking at this system, skip it.
					if ($bVault && ($systemID == $localID)) {
						continue;
					}
					
					// If this system is a manager, only look at the specified dpuID.
					if (!$bVault && $bMgr && ($systemID != $dpuID)) {
						continue;
					}
					
					$local = false;
					// if the id is the same as the local system, it's the local.
					if ($systemID == $localID || $systemID == $crossVaultDpuID) {
						$local = true;
					}
					
					$shouldGetHistory = shouldGetHistory($BP, $action, $local, $systemID);
					if ($shouldGetHistory === false) {
						continue;
					}
					
					// If we cannot get the list of clients, skip it.
					$clients = $BP->get_client_list($systemID);
					if ($clients === false) {
						continue;
					}
								
					$result_format = array_merge($originalFormat, array('system_id' => $systemID));			
					$backupStatusArray = $BP->get_backup_status($result_format);
							
					if ($backupStatusArray !== false) {		
						if (count($backupStatusArray) > 0) {
						
							// Sort the backups based on the compareDates function.
							usort($backupStatusArray, "compareDates");
							
							// Filter down to a single DPUstate or Appliance agent metadata for each report.
							// For action sync, show 1 dpuState.  For replicate, show 1 appliance metadata.
							$dpuState = $action == 'sync' ? false : true;
							$appMetaState = $action == 'replicate' ? false : true;
							
							foreach ($backupStatusArray as $backupStatus) {
								// get client name from grandclients for replicated backups
								if (isset($backupStatus['client_id'])) {
									$clientID = $backupStatus['client_id'];
									if ($action == "replicate" && isset($backupStatus['replicated']) && $backupStatus['replicated'] == true) {
										// set client name as "" for replicated backups
										$clientName = "";
									}
									else {
										$clientName = isset($clients[$clientID]) ? $clients[$clientID] : "unknown";
									}
								} else {
									$clientID = -1;
									$clientName = "unknown";
								}
								
								// get the instance and target for top panel.
								if ($action == 'replicate') {
									if (isset($backupStatus['database_name'])) {
										$instance = $backupStatus['database_name'];
									}
									else if (isset($backupStatus['vm_name'])) {
										$instance = $backupStatus['vm_name'];
									}
									else if (isset($backupStatus['oracle_sid'])) {
										$instance = $backupStatus['oracle_sid'];
									}
									else if (isset($backupStatus['server_instance_name'])) {
										$instance = $backupStatus['server_instance_name'];
									}
									else {
										$instance = FILE_LEVEL;
									}
									
									$backupStatus['Rep_Instance'] = $instance;
									$backupStatus['Rep_Target'] = isset($backupStatus['destination']) ? $backupStatus['destination'] :
																										"unknown";
								}
								
								if ($appMetaState && $backupStatus['type'] == $APP_METADATA ||
									$dpuState && $backupStatus['type'] == $DPUSTATE) {
									;
								} else {
									createBaseBackupStatusXML($xml, $backupStatus, NULL,
															  $systemID, $systemName, $clientID, $clientName, $tag);
								}

								if ($backupStatus['type'] == $APP_METADATA) {
									$appMetaState = true;
								}
								if ($backupStatus['type'] == $DPUSTATE) {
									$dpuState = true;
								}
							}
						}
					} else {
						createDPUErrorXML($xml, $BP, $systemName);
					}
				}
			}
			$xml->pop(); // BackupInformation
		} else {
			$BP->buildResult($xml, false);
		}

		echo($xml->getXml());
		break;
		
	case "check":
		$systems = $BP->get_system_list();
		if ($systems !== false) {
		
			$xml->push('root');
			foreach ($systems as $systemID => $systemName) {
				if ($systemID == $dpuID || $dpuID == $localID) {					
					// If this system is a vault and we are looking at this system, skip it.
					if ($bVault && ($systemID == $localID)) {
						continue;
					}
					
					// If this system is a manager, only look at the specified dpuID.
					if (!$bVault && $bMgr && ($systemID != $dpuID)) {
						continue;
					}
								
					$xml->push('DPU', array('ID' => $systemID, 'Name' => $systemName));	
					$result = $BP->get_securesync_config($systemID);
					if ($result !== false) {
						if (array_key_exists('AutoSyncEnabled', $result)) {
							$xml->element('AutoSyncEnabled', $result['AutoSyncEnabled'] == true ? '1' : '0');
						}
						if (array_key_exists('CheckIntervalSeconds', $result)) {
							$xml->element('CheckIntervalSeconds', $result['CheckIntervalSeconds']);
						}
					} else {
						// Keep going, but log the error.
						global $Log;
						$Log->writeError($BP->getError(), true);
					}
					
					// bug 16948 - LD, fix command so that vaulting dashboard shows 
					// the last and the next securesync iteration
					$result = $BP->run_command("Securesync Date for Vaulting Systems", "", $systemID);
					
					// if failed retry using the old command
					if ($result === false) {
						$result = $BP->run_command("Securesync Date", "", $systemID);
					}
					
					if ($result !== false) {
						$xml->push('SecuresyncDate');
						$xml->cData($result);
						$xml->pop();
					} else {
						$xml->push('SecuresyncDate');
						$xml->cData("Last Securesync Iteration: unknown");
						$xml->pop();
	
						// Keep going, but log the error.
						global $Log;
						$Log->writeError($BP->getError(), true);
					}
					$xml->pop(); // DPU
				}
			}
			$xml->pop(); // BackupInformation
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;

	case "eligible":
		if (isset($_GET['client']) && isset($_GET['btype'])) {
			// don't pass down bid if not set or not valid.
			$backupID = (isset($_GET['bid']) && (int)$_GET['bid'] > 0) ? (int)$_GET['bid'] : -1;
			$clientName = $_GET['client'];
			$backupType = getTypeForDetails(true, ($_GET['btype']));
			$result = $BP->is_eligible_for_reset($backupID, $clientName, $backupType, $dpuID);
			if ($result !== -1) {
				$xml->push("root");
				$xml->element('Reset', $result == true ? "1" : "0");
				$xml->pop(); // root
			} else {
				$BP->buildResult($xml, false);
			}
		} else {
			$stringError = sprintf("vaulting reset eligibility error: must specify client name and backup type.");
			$BP->buildResult($xml, false, $stringError);
		}
		echo($xml->getXml());
		break;
		
	case "reset":
		if (isset($_GET['client']) && isset($_GET['btype'])) {
			// don't pass down bid if not set or not valid.
			$backupID = (isset($_GET['bid']) && (int)$_GET['bid'] > 0) ? (int)$_GET['bid'] : -1;
			$clientName = $_GET['client'];
			$backupType = getTypeForDetails(true, ($_GET['btype']));
			$BP->buildResult($xml, $BP->reset_vaulting_operation($backupID, $clientName, $backupType, $dpuID));
		} else {
			$stringError = sprintf("vaulting reset argument error: must specify client name and backup type.");
			$BP->buildResult($xml, false, $stringError);
		}
		echo($xml->getXml());
		break;
}

//
// This function displays an error message for the row if the DPU is not accessible.
function createDPUErrorXML($xml, $BP, $dpuName)
{

	global $Log;
	$message = " Could not get backup information for dpu: " . $dpuName;
	$message = $BP->getError() . ":" . $message;
	$Log->writeError($message, true);
							
	$xml->push("Backup", array('Type' => '', "Identification" => '', "DPUName" => $dpuName, "ClientName" => '{system offline}'));
	$xml->element('Date', '');
	$xml->element('Time', '');
	$xml->pop(); // Backup
}

// If we are getting replication history, skip systems that aren't capable of replication or that are vaulting
// For a dpu personality, get history as long as it's not vaulting to get DRs
// If we are getting vaulting history, skip systems that aren't vaulting
// For a vault, get history for systems that are vaulting/replicating to this system and not elsewhere
// Dpu personality counts as a local system
function shouldGetHistory($BP, $type, $isLocalSystem, $systemID) {
	Global $bVault;
	Global $dpuID; 	// system that was passed in the http request
	Global $localID;
	
	$status = getReplicationAndVaultingStatus($systemID, $isLocalSystem);
	
	$result = false;
	
	// if getting aggregate on a vault, return false for systems that are only managed even
	// though they are replicating, unless it's a cross-vault
	if ($bVault && $dpuID == $localID ) {
		// on a vault and getting aggregate
		if (!$isLocalSystem && $status['isOnlyManaged'] == 1) {
			// not a local system/dpu personality and is only managed by this system
			return false;
		}
	}
	
	if ($type == "replicate") {
		// return false if system is vaulting
		$result = $status['vaulting'] === true ? false : true;
	}
	else if ($type == "sync") {
		// return false if system is replicating
		$result = $status['replicating'] === true ? false : true;
	}
	
	return $result;
}

$BP->destroyCookie();

?>
