<?php
// basic sequence with LDAP is connect, bind, search, interpret search
// result, close connection

class LDAPFunctions
{
	function __construct()
	{
	}
		
	function authenticate($username, $password, $fqdn, $ADssl) {
		$bSuccess = true;
		//build url
		$ldapURL = $ADssl ? "ldaps://" : "ldap://";
		$port = $ADssl ? ":636" : ":389";
		$sName = $this->getName($username);

		//turn on debug
	//	ldap_set_option(NULL, LDAP_OPT_DEBUG_LEVEL, 7);
		$ldaphost = $ldapURL . $fqdn . $port;

		$ds=ldap_connect($ldaphost);  // must be a valid LDAP server!
		
		if ($ds) {
			$this->bindLDAP($ds, &$r, $username, $password);
			
			if(!$r) {
				// try connecting using the ip address, if fqdn is resolvable
				ldap_close($ds);
				$ip = $this->getAddrByHost($fqdn);
				if ($ip != $fqdn) {
					// use ip address to create a new connection
					$ldaphost = $ldapURL . $ip . $port;
					$ds=ldap_connect($ldaphost);
					if ($ds) {
						$this->bindLDAP($ds, &$r, $username, $password);
					}
				}
				
				// if still unsuccessful, return an error
				if(!$r) {
					return "LDAP Bind failed.\n     Verify Active Directory name is correct in master.ini, \n     FQDN is present for server in host file or is resolvable through DNS,\n     and server is running. \n     Verify log in credentials.\n     This message can be disregarded if user is a System user.";
				}
			}
			
			$dn = $this->parseFQDN($fqdn);

		   $filter = "(&(objectCategory=user)(sAMAccountName=" . $sName . "))"; 
		   $attr=array("memberof" );

			$sr=ldap_search($ds, $dn, $filter, $attr);
			if(!$sr) {
				return "Failed to search";
			}else {
						
				$entry = ldap_first_entry($ds, $sr);
				$attrs = ldap_get_attributes($ds, $entry);
				
				$info = ldap_get_entries($ds, $sr);
			}
		
			ldap_close($ds);
		
		} else {
			return "not a valid host";
		}
		$info['rrclogin'] = $sName;
		
		return $info;
	}
	
	function parseFQDN($fqdn){
		$fqdn = str_replace(".", ",DC=", $fqdn);
		$dn = "DC=" . $fqdn;
		
		return $dn;
	}
	
	function getName($username) {
		if (strpbrk($username, '@') != FALSE) {
			$s = explode('@', $username);
			$name = $s[0];
		} elseif (strpbrk($username, '\\') != FALSE) { 
			$s = explode('\\', $username);
			$name = $s[1];
		} 
		if(isset($name)) {	
			return $name;
		}
	}
	
	function bindLDAP($ds, &$r, $username, $password) {
		//Set protocol version
		ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, 3);

		// Set this option for AD on Windows Server 2003 + per PHP manual
		ldap_set_option($ds, LDAP_OPT_REFERRALS, 0);

		$r=ldap_bind($ds, $username, $password);
	}

	// returns ip address if found, otherwise returns the 
	function getAddrByHost($fqdn, $timeout = 3) {
   		$query = `nslookup -timeout=$timeout -retry=1 $fqdn`;
   		if(preg_match('/\nAddress: (.*)\n/', $query, $matches))
      			return trim($matches[1]);
   		return $fqdn;
	}
}

?> 
