<?php
# handles logging to a specified file

$log_max_number = 4; // {app}_0.log through {app}_#.log
global $log_level;
global $log_filename;
global $log_handle;
global $log_is_open;

/**
 * @desc - initializes log $logname (e.g. statussync), rotates files
 *         if necessary, looks up log level, and prepares log for
 *         writing.  Log will be $logname_#.log
 * @param $logname - log's base name
 * @return - true on success, error string on error
 */
function initialize_log($logname)
{
	global $log_max_number;
	global $log_level;
	global $log_filename;
	global $log_handle;
	global $log_is_open;

	// get log level, log path
	// master.ini stuff
	$cmd_bppath = 'eval `grep BPDIR /etc/default/bp.ini`';
	$cmd_master_ini = 'MASTER_INI=$BPDIR/bpinit/master.ini';
	$cmd_bputil = 'BPUTIL=$BPDIR/bin/bputil';
	$cmd_ini = "$cmd_bppath; $cmd_master_ini; $cmd_bputil";
	$cmd_log_level = '$BPUTIL -g "debugging" "'.$logname.'" "0" $MASTER_INI';
	$cmd_log_path  = '$BPUTIL -g "Location Information" "Logs-Dir" "" $MASTER_INI';
	$log_level = trim(shell_exec("$cmd_ini; $cmd_log_level"));
	$log_path = trim(shell_exec("$cmd_ini; $cmd_log_path"));

	// manage file
	$log_filename = $log_path . '/' . $logname.'_0.log';
	if ($log_is_open) {
		fclose($log_handle);
	}
	$log_is_open = false;

	// rotate files if needed
	$date_string = date('F j, Y');
	$log_preamble = 'Log for '.$logname.', created '.$date_string."\n";
	if (is_dir($log_filename)) {
		return "specified log file '$log_filename' is a directory!";
	}
	if (is_writable($log_filename)) {
		$log_handle = fopen($log_filename, 'r');
		if ($log_handle === false) {
			return "could not open logfile '$log_filename'";
		}
		if (fgets($log_handle, 1024) != $log_preamble) {
			$rotate = true;
		} else {
			$rotate = false;
		}
		fclose($log_handle);
	} else {
		$rotate = false;
	}

	if ($rotate == true) {
		$logfiles = array(); // log filenames (no path)
		$dh = @opendir($log_path);
		if ($dh === false) {
			return "could not open directory '$log_path'";
		}
		while (false !== ($filename = readdir($dh))) {
			if (preg_match('/^' . $logname . '_\d\.log$/', $filename)) { // match log name
				$lognum = (int)substr($filename, strlen($logname)+1, -4); // cut off $logname_ from start, .log from end
				$logfiles[$lognum] = $filename;
			}
		}
		closedir($dh);
		// scan for any missing files, we won't have to move them all then
		$found_missing_log = false;
		$missing_log_num = -1;
		for ($i = 0; $i < $log_max_number; $i++) {
			if (!isset($logfiles[$i])) {
				$found_missing_log = true;
				$missing_log_num = $i;
			}
		}
		if ($found_missing_log == true) {
			$lowest_delete_num = $log_max_number + 1; // move some files, so no deleteions necessary
			$highest_move_num = $missing_log_num - 1; // work down from here to rename log files from app_$num.log to app_($num+1).log
		} else {
			$lowest_delete_num = $log_max_number; // moving all files, so remove the one before the max
			$highest_move_num = $log_max_number - 1; // move all files up
		}

		// handle deletions
		foreach ($logfiles as $num => $name) {
			if ($num >= $lowest_delete_num) {
				unlink($log_path . '/' . $name);
			}
		}

		// now rotate the files
		for ($i = $highest_move_num; $i >= 0; $i--) {
			if (!isset($logfiles[$i])) {
				continue; // for safety, this shouldn't happen
			}
			$oldname = $log_path . '/' . $logfiles[$i];
			$newname = $log_path . '/' . $logname . '_' . ($i+1) . '.log';
			if (false === rename($oldname, $newname)) {
				return "could not rename logfile '$oldname' to '$newname'";
			}
		}
	}

	// open file, write preamble if file is new
	$write_preamble = (is_file($log_filename) ? false : true);
	$handle = fopen($log_filename, 'a'); // open/create file for appending
	if ($handle === false) {
		return "could not open logfile '$log_filename' for writing";
	}
	if ($write_preamble) {
		fwrite($handle, $log_preamble);
	}
	$log_handle = $handle;

	// return success
	return true;
}



/**
 * @desc - writes to the open/initialized logfile
 * @note - semantics are different for this, a newline is appended automatically
 *         because otherwise you have to double quote long strings or append a
 *         newline, causing headaches for you or for php's string parser...
 * @precondition - log has been initialized
 * @param $level - log level (0-5), 0 = always log, 5 = log only for debugging
 * @param $text - text to log
 * @param optional $newline - false to exclude a newline, default is to append one
 * @return - true on success, false on error
 */
function logger($level, $text, $newline = true)
{
	global $log_handle;
	global $log_level;
	if ($level > $log_level) {
		return true;
	}
	if (!$log_handle) {
		return false;
	}
	if ($newline) {
		$bytes = fwrite($log_handle, $text."\n");
	} else {
		$bytes = fwrite($log_handle, $text);
	}
	return ($bytes !== false);
}

